package com.duben.roseplaylet.ad.splash

import android.app.Activity
import android.view.View
import android.widget.FrameLayout
import com.bytedance.sdk.openadsdk.*
import com.bytedance.sdk.openadsdk.mediation.MediationConstant
import com.bytedance.sdk.openadsdk.mediation.ad.MediationAdSlot
import com.bytedance.sdk.openadsdk.mediation.ad.MediationSplashRequestInfo
import com.duben.roseplaylet.BuildConfig
import com.duben.roseplaylet.MintsApplication
import com.duben.roseplaylet.ad.AdStatusListener
import com.duben.roseplaylet.common.Constant
import com.duben.roseplaylet.manager.TrackManager
import com.duben.roseplaylet.utils.LogUtil
import com.duben.roseplaylet.utils.UIUtils
import java.lang.ref.WeakReference


/**
 * 应用内-网赚开屏
 */
object SplashManager {

    // 兜底配置
    val pangleSplashBottom = object :
        MediationSplashRequestInfo(
            MediationConstant.ADN_PANGLE, "889457860",
            BuildConfig.GROMORE_APP_ID, ""
        ) {} //ok

    private val TAG = SplashManager::class.java.simpleName

    private var mTTSplashAd: CSJSplashAd? = null

    private const val AD_TIME_OUT = 5000 //开屏广告加载超时时间
    private var isLoadSuccess = 0 //0-加载失败/未加载 1-加载中 2-加载成功
    private var isShowAd = false // 是否显示广告
    private var adStatusListener: AdStatusListener? = null

    /**
     * 预加载开屏广告
     */
    fun preLoadAd(activity: Activity) {
        val weakActivity = WeakReference<Activity>(activity)
        LogUtil.d(TAG, "gromore 开屏广告-> 1、进入预加载")
        if (isLoadSuccess == 1) {

            return
        }
        val adNativeLoader = TTAdSdk.getAdManager().createAdNative(weakActivity.get())
        val adslot = AdSlot.Builder()
            .setCodeId(BuildConfig.GROMORE_SPLASH_CODE)
            .setImageAcceptedSize(
                UIUtils.getScreenWidth(MintsApplication.getContext()),
                UIUtils.getScreenHeight(MintsApplication.getContext())
            )
            .setMediationAdSlot(
                MediationAdSlot.Builder()
                    .setMediationSplashRequestInfo(pangleSplashBottom)
                    .build()
            )
            .build()

        adNativeLoader.loadSplashAd(adslot, object : TTAdNative.CSJSplashAdListener {
            override fun onSplashLoadSuccess(p0: CSJSplashAd?) {
            }

            override fun onSplashLoadFail(p0: CSJAdError?) {
                LogUtil.e(
                    TAG,
                    "gromore 开屏广告--> 3、Gromore  onSplashLoadFail  adErrorCode=${p0?.code}  "
                )
                isLoadSuccess = 0
            }

            override fun onSplashRenderSuccess(p0: CSJSplashAd?) {
                LogUtil.d(TAG, "gromore 开屏广告--> 3、Gromore  onSplashAdLoadSuccess")
                LogUtil.d("SplashNewActivity-->", "333333--> ad show")

                isLoadSuccess = 2
                mTTSplashAd = p0
            }

            override fun onSplashRenderFail(p0: CSJSplashAd?, p1: CSJAdError?) {
                LogUtil.e(
                    TAG,
                    "gromore 开屏广告--> 3、Gromore  onSplashRenderFail  adErrorCode=${p1?.code}  "
                )
                isLoadSuccess = 0
            }

        },AD_TIME_OUT)
    }

    /**
     * 展示开屏广告
     */
    fun showAd(activity: Activity, view: FrameLayout, adStatusListener: AdStatusListener?) {
        this.adStatusListener = adStatusListener
        isShowAd = false
        if (mTTSplashAd != null && isLoadSuccess == 2) {
            LogUtil.d(TAG, "gromore 开屏广告--> 准备调用show")
            mTTSplashAd?.let {
                it.setSplashAdListener(adInteractionListener)
                view.addView(it.splashView)
            }
        } else {
            LogUtil.d(
                TAG,
                "gromore 开屏广告--> 准备调用show 失败 mTTSplashAd=${mTTSplashAd} isLoadSuccess=${isLoadSuccess}"
            )
            adStatusListener?.adFail()
        }
        // 状态重置
        isLoadSuccess = 0
    }

    /**
     * 广告是否加载成功
     */
    fun isSuccessStatus() = isLoadSuccess == 2

    val adInteractionListener = object : CSJSplashAd.SplashAdListener {
        override fun onSplashAdShow(p0: CSJSplashAd?) {
            isShowAd = true
            LogUtil.d(TAG, "gromore 开屏广告--> 6、onAdShow ")

            mTTSplashAd?.let {
                if (it.mediationManager != null && it.mediationManager.showEcpm != null&& it.mediationManager.showEcpm.ecpm != null) {
                    val vo = HashMap<String, Any>()
                    vo["cpmFen"] = it.mediationManager.showEcpm.ecpm
                    vo["adType"] = Constant.GRO_MORE_ADTYPE0
                    TrackManager.getInstance().cmtEcpm(vo)
                }
            }
        }

        override fun onSplashAdClick(p0: CSJSplashAd?) {
            LogUtil.d(TAG, "gromore 开屏广告--> 7、onAdDismiss ")
//            adStatusListener?.adClose(null)
        }

        override fun onSplashAdClose(p0: CSJSplashAd?, p1: Int) {
            LogUtil.d(TAG, "gromore 开屏广告--> 7、onAdDismiss ")
            adStatusListener?.adClose(null)
        }
    }

    /**
     * 是否展示广告
     */
    fun getAdStatus() = isShowAd

}