package com.duben.roseplaylet.manager

import android.annotation.SuppressLint
import android.app.Activity
import android.os.Handler
import android.os.Message
import android.text.TextUtils
import com.alipay.sdk.app.AuthTask
import com.duben.roseplaylet.MintsApplication
import com.duben.roseplaylet.common.AppConfig
import com.duben.roseplaylet.mvp.model.AuthResult
import com.duben.roseplaylet.mvp.model.BaseResponse
import com.duben.roseplaylet.utils.ToastUtil
import com.duben.library.net.neterror.BaseSubscriber
import com.duben.library.net.neterror.Throwable
import com.duben.library.utils.json.JsonUtil
import com.google.gson.JsonObject
import java.lang.ref.WeakReference

/**
 * 支付宝授权
 */
object AlipayAuthManager {

    private const val SDK_AUTH_FLAG = 2

    @SuppressLint("HandlerLeak")
    private val mHandler: Handler = object : Handler() {
        override fun handleMessage(msg: Message) {
            when (msg.what) {
                SDK_AUTH_FLAG -> {
                    val authResult = AuthResult(msg.obj as Map<String, String>, true)
                    val resultStatus: String = authResult.getResultStatus()
                    // 判断resultStatus 为“9000”且result_code
                    // 为“200”则代表授权成功，具体状态码代表含义可参考授权接口文档
                    if (TextUtils.equals(
                            resultStatus,
                            "9000"
                        ) && TextUtils.equals(authResult.getResultCode(), "200")
                    ) {
                        // 获取alipay_open_id，调支付时作为参数extern_token 的value
                        // 传入，则支付账户为该授权账户
                        println("mcg -->>>>>  authinfo=" + JsonUtil.toJson(authResult))
                        commitAlipayAutoInfo(JsonUtil.toJson(authResult))
                    } else {
                        // 其他状态值则为授权失败
                        authListener?.authFail(resultStatus)
                    }
                }
                else -> {}
            }
        }
    }

    private fun commitAlipayAutoInfo(info: String) {
        val vo = HashMap<String, Any>()
        vo.put("user_id", UserManager.getInstance().userID)
        vo.put("result", info)
        val baseApplication = MintsApplication.getContext() as MintsApplication
        AppHttpManager.getInstance(baseApplication)
            .call(baseApplication.loanService.reportAlipayMsg(vo),
                object : BaseSubscriber<BaseResponse<Any>>() {
                    override fun onError(e: Throwable) {
                        authListener?.authFail("授权失败！")
                    }

                    override fun onNext(t: BaseResponse<Any>) {
                        if (t.status == 200) {
//                            UserManager.getInstance().alipaySet = true
                            authListener?.authSuccess()
                        } else {
                            authListener?.authFail(t.message)
                        }

                    }
                })
    }

    /**
     * 支付宝账户授权业务示例
     */
    fun authV2(activity: Activity, listener: AuthListener?) {
        authListener = listener
        val weakReference = WeakReference(activity)
        println("mcg -->>>>>  开始授权")
        val vo = HashMap<String, Any>()
        vo.put("user_id", UserManager.getInstance().userID)
        val baseApplication = MintsApplication.getContext() as MintsApplication
        AppHttpManager.getInstance(baseApplication)
            .call(baseApplication.loanService.getAlipayAuthUrl(vo),
                object : BaseSubscriber<BaseResponse<JsonObject>>() {
                    override fun onError(e: Throwable) {

                    }

                    override fun onNext(t: BaseResponse<JsonObject>) {
                        if (t.status == 200) {
                            val data = t.data
                            val authInfo = data["url"].asString
                            val authRunnable = Runnable { // 构造AuthTask 对象
                                val authTask = AuthTask(weakReference.get())
                                // 调用授权接口，获取授权结果
                                val result = authTask.authV2(authInfo, true)
                                val msg = Message()
                                msg.what = SDK_AUTH_FLAG
                                msg.obj = result
                                mHandler.sendMessage(msg)
                            }

                            // 必须异步调用
                            val authThread = Thread(authRunnable)
                            authThread.start()
                        } else {
                            ToastUtil.show(MintsApplication.getContext(), t.message)
                        }
                    }
                })
    }

    private var authListener: AuthListener? = null

    interface AuthListener {
        fun authSuccess()
        fun authFail(resultStatus: String)
    }
}