package com.duben.roseplaylet.manager

import android.app.Application
import android.widget.Toast
import com.bytedance.sdk.djx.DJXSdk
import com.bytedance.sdk.djx.DJXSdkConfig
import com.bytedance.sdk.djx.IDJXService
import com.bytedance.sdk.djx.model.DJXDrama
import com.bytedance.sdk.djx.model.DJXError
import com.bytedance.sdk.djx.model.DJXOthers
import com.duben.library.utils.json.JsonUtil
import com.duben.roseplaylet.BuildConfig
import com.duben.roseplaylet.MintsApplication
import com.duben.roseplaylet.mvp.model.HotStyleTypesBean
import com.duben.roseplaylet.mvp.model.VedioBean
import com.duben.roseplaylet.utils.LogUtil

/**
 * 描述：穿山甲短剧
 *
 * 作者：孟崔广
 */
object CsjDJXSdkManager {

    private const val TAG = "CsjDJXSdkManager"

    /**
     * 短剧初始化
     *
     * 注意：穿山甲广告sdk初始成功后使用
     */
    fun init(application: Application) {
        //初始化，最好放到application.onCreate()执行
        val config = DJXSdkConfig.Builder().build()
        DJXSdk.init(application, "SDK_Setting_5458827.json", config)

    }
    fun start(application: Application) {

        DJXSdk.start(object : DJXSdk.StartListener {
            override fun onStartComplete(p0: Boolean, p1: String?, p2: DJXError?) {
                LogUtil.d(TAG, "p0 = $p0--p1 = $p1--p2 = ${JsonUtil.toJson(p2)}")
            }
        })
    }

    /**
     * 短剧分类
     */
    fun requestDramaCategoryList(listener: CsjdramaCategoryListListener) {
        if (DJXSdk.isStartSuccess()) {
            DJXSdk.service()
                .requestDramaCategoryList(object : IDJXService.IDJXCallback<MutableList<String>?> {
                    override fun onError(error: DJXError) {
                        LogUtil.d(TAG, "request failed, code = $error")
                        listener.dramaCategoryListFail()
                    }

                    override fun onSuccess(
                        categoryList: MutableList<String>?,
                        others: DJXOthers?
                    ) {
                        val hotStyleTypesBeanList = mutableListOf<HotStyleTypesBean>()
                        categoryList?.let { list ->
                            list?.forEach { data ->
                            }
                            val categoty = list.listIterator()
                            while (categoty.hasNext()) {
                                val HotStyleTypesBean = HotStyleTypesBean()
                                val typeId =categoty.nextIndex()
                                val type =categoty.next()
                                if (BuildConfig.DEBUG) {
                                    LogUtil.d(
                                        TAG,
                                        "Index: ${typeId}, Category: ${type}"
                                    )
                                }
                                HotStyleTypesBean.typeId = typeId
                                HotStyleTypesBean.type = type
                                HotStyleTypesBean.torder = typeId
                                hotStyleTypesBeanList.add(HotStyleTypesBean)
                            }
                            listener.dramaCategoryListSuccess(hotStyleTypesBeanList)
                        }
                    }
                })

        } else {
            listener.dramaCategoryListFail()
        }
    }

    interface CsjdramaCategoryListListener {
        fun dramaCategoryListSuccess(list: MutableList<HotStyleTypesBean>)
        fun dramaCategoryListFail()
    }

    /**
     * 按分类查询短剧
     */
    fun requestDramaByCategory(
        category: String,
        page: Int,
        size: Int,
        listener: CsjrequestDramaByCategoryListener
    ) {
        if (DJXSdk.isStartSuccess()) {
            val order = 0// 0是正序 1是倒序
            DJXSdk.service().requestDramaByCategory(
                category,
                page,
                size,
                order,
                object : IDJXService.IDJXCallback<MutableList<out DJXDrama>?> {
                    override fun onError(error: DJXError) {
                        LogUtil.d(
                            TAG,
                            "drama category: request failed, code = $error"
                        )
                        listener.dramaCategoryListFail()
                    }

                    override fun onSuccess(
                        dataList: MutableList<out DJXDrama>?,
                        others: DJXOthers?
                    ) {
                        val vedioList = mutableListOf<VedioBean>()
                        LogUtil.d(TAG, "request success, drama size = " + dataList?.size)
                        dataList?.forEach { drama ->
                            LogUtil.d(TAG, "drama category: $drama")
                            //{freeSet=0, lockSet=1, reqId='202501071912576331796EC2409C457B02', groupId=0, providerId=2, channelId=0, rankId=1, visibility=false, id=1042, title='重回90年代', coverImage='https://p3-csj-sign.byteimg.com/tos-cn-i-4g66r8cj84/1c0eca384bef41569220a05b18b30976~tplv-4g66r8cj84-jpeg.image?rk3s=363bf5d8&x-expires=1736853177&x-signature=rCgxWR0dK60b2cLh9%2B527%2Be%2F7PA%3D', coverImages2='null', status=0, total=59, index=1, type='悬疑', typeId=4, desc='彭显光遭遇小人陷害，家破人亡，妻离子散；同时他意外穿越回1990年，遇到了已故的妻女，他发誓这辈子一定要好好守护妻女，抓住幕后的恶人，不会让历史重蹈覆辙！', scriptName='', scriptAuthor='', createTime=1693405348', actionTime=0', unlockIndex=0', episodeStatuses=null', icpNumber=1108324230306040}
                            val vedioBean = VedioBean()
                            vedioBean.completeStatus = 0
                            vedioBean.title = drama.title
                            vedioBean.vedioDesc = drama.desc
                            vedioBean.coverImage = drama.coverImage
                            vedioBean.thirdId = drama.id.toString()
                            // 设置短剧的免费集数
                            vedioBean.freeIndex = drama.freeSet
                            // 设置短剧广告解锁剧贪杯
                            vedioBean.unlockIndex = drama.lockSet
                            vedioList.add(vedioBean)
                        }
                        listener.dramaCategoryListSuccess(vedioList)
                    }
                })
        } else {
            listener.dramaCategoryListFail()
        }
    }

    interface CsjrequestDramaByCategoryListener {
        fun dramaCategoryListSuccess(list: MutableList<VedioBean>)
        fun dramaCategoryListFail()
    }

    /**
     * 短剧详情
     */
    fun requestDrama(id: Long, callback: (DJXDrama) -> Unit) {
        DJXSdk.service().requestDrama(
            listOf(id),
            object : IDJXService.IDJXCallback<MutableList<out DJXDrama>?> {
                override fun onError(error: DJXError) {
                    Toast.makeText(MintsApplication.getContext(), "请求失败", Toast.LENGTH_SHORT)
                        .show()
                    LogUtil.d(TAG, "request failed, code = $error")
                }

                override fun onSuccess(
                    dataList: MutableList<out DJXDrama>?,
                    others: DJXOthers?
                ) {
                    dataList?.takeIf { it.isNotEmpty() }?.first()?.let {
                        //{freeSet=0, lockSet=1, reqId='20250108144948E0223379C7C0B34AB595', groupId=0, providerId=28, channelId=0, rankId=1, visibility=false, id=6187, title='荒楼诡事', coverImage='https://p6-csj-sign.byteimg.com/tos-cn-i-4g66r8cj84/650229886c2b4c44a0caf4342ccbe33e~tplv-4g66r8cj84-size:1000000.image?rk3s=363bf5d8&x-expires=1736923788&x-signature=c5Qnc%2F1KSmz5Hv2zzxSgvS5MFtk%3D', coverImages2='[DJXImage{url='https://p9-csj-sign.byteimg.com/tos-cn-i-4g66r8cj84/650229886c2b4c44a0caf4342ccbe33e~tplv-4g66r8cj84-image.image?rk3s=363bf5d8&x-expires=1736923788&x-signature=79ZTGJbUM7PclM3ZPkStGYj01%2BA%3D', backupUrl='https://p6-csj-sign.byteimg.com/tos-cn-i-4g66r8cj84/650229886c2b4c44a0caf4342ccbe33e~tplv-4g66r8cj84-image.image?rk3s=363bf5d8&x-expires=1736923788&x-signature=4TS9Xqs9PNM4ijwzudbNAKnkHSE%3D', definition='original'}, DJXImage{url='https://p9-csj-sign.byteimg.com/tos-cn-i-4g66r8cj84/650229886c2b4c44a0caf4342ccbe33e~tplv-4g66r8cj84-maxresize:540:10000:q80:1000000.image?rk3s=363bf5d8&x-expires=1736923788&x-signature=oSlF3QAC0R%2BpVlXM%2FSpj8tzWcXc%3D', backupUrl='https://p6-csj-sign.byteimg.com/tos-cn-i-4g66r8cj84/650229886c2b4c44a0caf4342ccbe33e~tplv-4g66r8cj84-maxresize:540:10000:q80:1000000.image?rk3s=363bf5d8&x-expires=1736923788&x-signature=Yt5nJhYdmE%2FyLrl4taZwLoJtVxo%3D', definition='540'}, DJXImage{url='https://p9-csj-sign.byteimg.com/tos-cn-i-4g66r8cj84/650229886c2b4c44a0caf4342ccbe33e~tplv-4g66r8cj84-maxresize:360:10000:q80:1000000.image?rk3s=363bf5d8&x-expires=1736923788&x-signature=4GNO43Mc4Ct%2FYhna%2FVgyCC6wADI%3D', backupUrl='https://p6-csj-sign.byteimg.com/tos-cn-i-4g66r8cj84/650229886c2b4c44a0caf4342ccbe33e~tplv-4g66r8cj84-maxresize:360:10000:q80:1000000.image?rk3s=363bf5d8&x-expires=1736923788&x-signature=TKJ%2FUqFGA6YUxDeMFupuUDoQ0R8%3D', definition='360'}]', status=0, total=100, index=1, type='悬疑', typeId=4, desc='西兰市近日发生3起女性奸杀的恶性连环杀人事件，近日的夜晚也因此而笼罩着诡异的氛围，一名长相靓丽的女人正走在偏僻的小道上，背后尾随着一位看起来人畜无害的纯情弟弟……', scriptName='', scriptAuthor='', createTime=1713952815', actionTime=0', unlockIndex=0', episodeStatuses=null', icpNumber=}
                        LogUtil.d(TAG, "request success, $it")
                        callback.invoke(it)
                    }
                }
            })
    }
}