package com.duben.roseplaylet.manager

import android.content.Context
import android.os.Bundle
import android.util.Log
import android.widget.Toast
import com.duben.roseplaylet.mvp.model.FaceParam
import com.duben.roseplaylet.mvp.model.FaceResult
import com.tencent.cloud.huiyansdkface.facelight.api.WbCloudFaceContant
import com.tencent.cloud.huiyansdkface.facelight.api.WbCloudFaceVerifySdk
import com.tencent.cloud.huiyansdkface.facelight.api.listeners.WbCloudFaceVerifyLoginListener
import com.tencent.cloud.huiyansdkface.facelight.api.result.WbFaceError
import com.tencent.cloud.huiyansdkface.facelight.process.FaceVerifyStatus

/**
 *  @author Assen
 *  @date 2025/1/9
 *  @desc
 */
class FaceVerifyHelper private constructor() {

    companion object {
        val instance: FaceVerifyHelper by lazy(mode = LazyThreadSafetyMode.SYNCHRONIZED) {
            FaceVerifyHelper()
        }

        const val TAG = "FaceVerifyHelper"
    }

    //拉起刷脸sdk
    fun openCloudFaceService(ctx: Context, param: FaceParam, callback: FaceVerifyCallback) {
        Log.d(TAG, "openCloudFaceService")
        val data = Bundle()
        val inputData: WbCloudFaceVerifySdk.InputData = WbCloudFaceVerifySdk.InputData(
            param.faceId,
            param.agreementNo,
            param.openApiAppId,
            param.openApiAppVersion,
            param.openApiNonce,
            param.openApiUserId,
            param.openApiSign,
            FaceVerifyStatus.Mode.GRADE,
            param.keyLicence
        )
        data.putSerializable(WbCloudFaceContant.INPUT_DATA, inputData)
        data.putString(WbCloudFaceContant.LANGUAGE, WbCloudFaceContant.LANGUAGE_ZH_CN)
        //颜色设置,sdk内置黑色和白色两种模式，默认白色
        //如果客户想定制自己的皮肤，可以传入WbCloudFaceContant.CUSTOM模式,此时可以配置ui里各种元素的色值
        //定制详情参考app/res/colors.xml文件里各个参数
        data.putString(WbCloudFaceContant.COLOR_MODE, WbCloudFaceContant.WHITE)
        //是否需要录制上传视频 默认不需要
        data.putBoolean(WbCloudFaceContant.VIDEO_UPLOAD, false)
        //是否播放提示音，默认不播放
        data.putBoolean(WbCloudFaceContant.PLAY_VOICE, false)
        //是否指定横屏，默认false，指定竖屏
        data.putBoolean(WbCloudFaceContant.IS_LANDSCAPE, false)
        //横竖屏是否跟随系统（仅限pad），默认false
        data.putBoolean(WbCloudFaceContant.IS_FOLLOW_SYSTEM, false)
        //识别阶段合作方定制提示语,可不传，此处为demo演示
        data.putString(WbCloudFaceContant.CUSTOMER_TIPS_LIVE, "仅供体验使用 请勿用于投产!")
        //上传阶段合作方定制提示语,可不传，此处为demo演示
        data.putString(WbCloudFaceContant.CUSTOMER_TIPS_UPLOAD, "仅供体验使用 请勿用于投产!")
        //合作方长定制提示语，可不传，此处为demo演示
        //设置选择的比对类型  默认为权威库对比
        //权威库比对 WbCloudFaceContant.ID_CRAD
        data.putString(WbCloudFaceContant.COMPARE_TYPE, WbCloudFaceContant.ID_CARD)
        //sdk log开关，默认关闭，debug调试sdk问题的时候可以打开,打开日志开关需要外部存储权限
        //【特别注意】上线前请务必关闭sdk log开关！！！
        data.putBoolean(WbCloudFaceContant.IS_ENABLE_LOG, true)
        Log.d(TAG, "WbCloudFaceVerifySdk initSdk")
        //【特别注意】请使用activity context拉起sdk
        //【特别注意】请在主线程拉起sdk！
        WbCloudFaceVerifySdk.getInstance()
            .initSdk(ctx, data, object : WbCloudFaceVerifyLoginListener {
                override fun onLoginSuccess() {
                    //登录sdk成功
                    Log.i(TAG, "onLoginSuccess")
                    //拉起刷脸页面
                    //【特别注意】请使用activity context拉起sdk
                    //【特别注意】请在主线程拉起sdk！
                    WbCloudFaceVerifySdk.getInstance().startWbFaceVerifySdk(ctx) { result ->
                        //得到刷脸结果
                        if (result != null) {
                            if (result.isSuccess) {
                                //刷脸成功可以去后台查询刷脸结果
                                Log.d(
                                    TAG,
                                    "刷脸成功! Sign=" + result.sign
                                        .toString() + "; liveRate=" + result.liveRate
                                        .toString() +
                                            "; similarity=" + result.similarity
                                        .toString() + "userImageString=" + result.userImageString
                                )
                                Toast.makeText(
                                    ctx,
                                    "刷脸成功",
                                    Toast.LENGTH_SHORT
                                ).show()


                                callback.onFaceVerifyOver(
                                    FaceResult(true, "", "", "", "")
                                )

                            } else {
                                val error: WbFaceError? = result.error
                                if (error != null) {
                                    Log.d(
                                        TAG, "刷脸失败！domain=" + error.domain
                                            .toString() + " ;code= " + error.code
                                            .toString() + " ;desc=" + error.desc
                                            .toString() + ";reason=" + error.reason
                                    )
                                    if (error.domain.equals(WbFaceError.WBFaceErrorDomainCompareServer)
                                    ) {
                                        //虽然对比失败，但这个domain也可以去后台查询刷脸结果
                                        Log.d(
                                            TAG, ("对比失败，liveRate=" + result.liveRate
                                                .toString() +
                                                    "; similarity=" + result.similarity)
                                        )
                                    }

                                    callback.onFaceVerifyOver(
                                        FaceResult(
                                            false,
                                            error.domain,
                                            error.code,
                                            error.desc,
                                            error.reason
                                        )
                                    )

                                    Toast.makeText(
                                        ctx, "刷脸失败!" + error.desc,
                                        Toast.LENGTH_LONG
                                    ).show()
                                } else {
                                    callback.onFaceVerifyOver(
                                        FaceResult(true, "", "", "", "")
                                    )

                                    Log.e(TAG, "sdk返回error为空！")
                                }
                            }
                        } else {
                            Log.e(TAG, "sdk返回结果为空！")
                        }
                        //测试用代码
                        //不管刷脸成功失败，只要结束了，都更新userId
                        Log.d(TAG, "更新userId")
//                        userId = "WbFaceVerifyREF" + System.currentTimeMillis()
                        //刷脸结束后，释放资源
                        WbCloudFaceVerifySdk.getInstance().release()
                    }
                }

                override fun onLoginFailed(error: WbFaceError?) {
                    //登录失败
                    Log.i(TAG, "onLoginFailed!")
                    if (error != null) {
                        Log.d(
                            TAG,
                            ("登录失败！domain=" + error.domain
                                .toString() + " ;code= " + error.code
                                .toString() + " ;desc=" + error.desc
                                .toString() + ";reason=" + error.reason)
                        )
                        if (error.domain.equals(WbFaceError.WBFaceErrorDomainParams)) {
                            Toast.makeText(
                                ctx,
                                "传入参数有误！" + error.desc,
                                Toast.LENGTH_SHORT
                            ).show()
                        } else {
                            Toast.makeText(
                                ctx,
                                "登录刷脸sdk失败！" + error.desc,
                                Toast.LENGTH_SHORT
                            ).show()
                        }

                        FaceResult(false, error.domain, error.code, error.desc, error.reason)
                    } else {
                        Log.e(TAG, "sdk返回error为空！")
                        callback.onFaceVerifyOver(
                            FaceResult(false, "", "", "", "")
                        )
                    }
                    //刷脸结束后，释放资源
                    WbCloudFaceVerifySdk.getInstance().release()
                }
            })
    }


    interface FaceVerifyCallback {
        fun onFaceVerifyOver(result: FaceResult)
    }

}