package com.duben.roseplaylet.mvp.presenters

import com.duben.library.net.neterror.BaseSubscriber
import com.duben.library.net.neterror.Throwable
import com.duben.roseplaylet.manager.AppHttpManager
import com.duben.roseplaylet.mvp.model.BaseResponse
import com.duben.roseplaylet.mvp.model.FaceParam
import com.duben.roseplaylet.mvp.model.FaceResult
import com.duben.roseplaylet.mvp.views.RealAuthView
import com.google.gson.JsonObject

class RealAuthPresenter : BasePresenter<RealAuthView>() {

    fun getFaceParam(imgUrl: String) {
        val vo = hashMapOf<String, Any>()
        vo["imgUrl"] = imgUrl
        AppHttpManager.getInstance(loanApplication)
            .call(loanService.getFaceParam(vo),
                object : BaseSubscriber<BaseResponse<FaceParam>>() {
                    override fun onCompleted() {
                        if (isLinkView) return
                        view.hideLoading()
                    }

                    override fun onNext(baseResponse: BaseResponse<FaceParam>) {
                        if (isLinkView) return

                        view.hideLoading()
                        val code = baseResponse.status
                        val message = baseResponse.message

                        when (code) {
                            200 -> view.getFaceParamSuc(baseResponse.data)
                            else -> {
                                view.showToast(message)
                                view.getFaceParamFail()
                            }
                        }
                    }

                    override fun onError(e: Throwable?) {
                        if (isLinkView) return
                        view.hideLoading()
                        view.showToast(e?.message)
                        view.getFaceParamFail()
                    }
                })
    }

    fun faceResult(faceResult: FaceResult) {
        val vo = hashMapOf<String, Any>()
        vo["success"] = faceResult.isSuccess
        vo["domain"] = faceResult.domain
        vo["code"] = faceResult.code
        vo["desc"] = faceResult.desc
        vo["reason"] = faceResult.reason
        AppHttpManager.getInstance(loanApplication)
            .call(loanService.getFaceResult(vo),
                object : BaseSubscriber<BaseResponse<JsonObject>>() {
                    override fun onCompleted() {
                        if (isLinkView) return
                        view.hideLoading()
                    }

                    override fun onNext(baseResponse: BaseResponse<JsonObject>) {
                        if (isLinkView) return

                        view.hideLoading()
                        val code = baseResponse.status
                        val message = baseResponse.message

                        when (code) {
                            200 -> view.faceResultSuc()
                            else -> {
                                view.showToast(message)
                                view.faceResultFail()
                            }
                        }
                    }

                    override fun onError(e: Throwable?) {
                        if (isLinkView) return
                        view.hideLoading()
                        view.showToast(e?.message)
                        view.faceResultFail()
                    }
                })
    }

}