package com.duben.roseplaylet.mvp.presenters

import com.duben.library.net.neterror.BaseSubscriber
import com.duben.library.net.neterror.Throwable
import com.duben.roseplaylet.manager.AppHttpManager
import com.duben.roseplaylet.mvp.model.BannerList
import com.duben.roseplaylet.mvp.model.BaseResponse
import com.duben.roseplaylet.mvp.views.SquareView

class SquarePresenter : BasePresenter<SquareView>() {

    fun autoList() {
        AppHttpManager.getInstance(loanApplication)
            .call(loanService.autoList(), object : BaseSubscriber<BaseResponse<BannerList>>() {
                override fun onCompleted() {
                    if (isLinkView) return
                    view.hideLoading()
                }

                override fun onNext(baseResponse: BaseResponse<BannerList>) {
                    if (isLinkView) return

                    view.hideLoading()
                    val code = baseResponse.status
                    val message = baseResponse.message

                    when (code) {
                        200 -> view.autoListSuc(baseResponse.data)
                        else -> {
                            view.autoListFail()
                            view.showToast(message)
                        }
                    }
                }

                override fun onError(e: Throwable?) {
                    if (isLinkView) return
                    view.hideLoading()
                    view.showToast(e?.message)

                    view.autoListFail()
                }

            })
    }


    fun getHallList(type: Int, page: Int, size: Int) {
        //1 推荐 2活跃 3新人
        val vo = hashMapOf<String, Any>()
        vo["type"] = type
        vo["page"] = page
        vo["size"] = size
        AppHttpManager.getInstance(loanApplication)
            .call(loanService.getHallList(vo), object : BaseSubscriber<BaseResponse<BannerList>>() {
                override fun onCompleted() {
                    if (isLinkView) return
                    view.hideLoading()
                }

                override fun onNext(baseResponse: BaseResponse<BannerList>) {
                    if (isLinkView) return

                    view.hideLoading()
                    val code = baseResponse.status
                    val message = baseResponse.message

                    when (code) {
                        200 -> view.getHallListSuc(baseResponse.data)
                        else -> {
                            view.getHallListFail()
                            view.showToast(message)
                        }
                    }
                }

                override fun onError(e: Throwable?) {
                    if (isLinkView) return
                    view.hideLoading()
                    view.showToast(e?.message)
                    view.getHallListFail()
                }

            })
    }


    fun topTabs() {
        AppHttpManager.getInstance(loanApplication)
            .call(loanService.topTabs(),
                object : BaseSubscriber<BaseResponse<BannerList>>() {
                    override fun onCompleted() {
                        if (isLinkView) return
                    }

                    override fun onError(e: Throwable) {
                        if (isLinkView) return
                    }

                    override fun onNext(baseResponse: BaseResponse<BannerList>) {
                        if (isLinkView) return

                        val code = baseResponse.status
                        val message = baseResponse.message

                        when (code) {
                            200 -> {
                                view.topTabsSuc(baseResponse.data)
                            }
                            else -> {
                                view.showToast(message)
                            }
                        }
                    }
                })
    }
}