package com.duben.roseplaylet.ui.activitys

import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.widget.TextView
import androidx.core.content.ContextCompat
import androidx.fragment.app.Fragment
import com.duben.roseplaylet.R
import com.duben.roseplaylet.ui.activitys.base.BaseActivity
import com.duben.library.utils.nodoubleclick.AntiShake
import com.duben.roseplaylet.ui.adapter.CommonTabAdapter
import com.duben.roseplaylet.ui.fragment.CommonListFragment
import com.google.android.material.tabs.TabLayout
import com.google.android.material.tabs.TabLayoutMediator
import kotlinx.android.synthetic.main.activity_common_tab.*
import kotlinx.android.synthetic.main.header_layout.*

/**
 * 描述：共用TAB页面
 *
 */
class CommonTabActivity : BaseActivity(), View.OnClickListener {

    companion object {
        const val ACTIVITY_COMMON_TYPE = "ACTIVITY_COMMON_TYPE"
        const val ACTIVITY_COMMON_POSITION = "ACTIVITY_COMMON_POSITION"

        // 意中人
        const val ACTIVITY_TYPE_YZR = "ACTIVITY_TYPE_TAB_YZR"

        // 浏览记录
        const val ACTIVITY_TYPE_LLJL = "ACTIVITY_TYPE_LLJL"

        // 联系记录
        const val ACTIVITY_TYPE_LXJL = "ACTIVITY_TYPE_LXJL"

        // 打招呼
        const val ACTIVITY_TYPE_DZH = "ACTIVITY_TYPE_DZH"

        // 站内信
        const val ACTIVITY_TYPE_ZNX = "ACTIVITY_TYPE_ZNX"

        // 通知
        const val ACTIVITY_TYPE_TZ = "ACTIVITY_TYPE_TZ"

        // 充值/消费记录
        const val ACTIVITY_TYPE_XFJL = "ACTIVITY_TYPE_XFJL"
    }

    private var mType = ACTIVITY_TYPE_YZR

    private val tabsData = mutableListOf<String>()
    private val fragments = mutableListOf<Fragment>()
    private var vpAdapter: CommonTabAdapter? = null
    private var mSelectTabIndex = 0

    override fun getContentViewLayoutID() = R.layout.activity_common_tab

    override fun isApplyKitKatTranslucency() = false

    override fun getBundleExtras(extras: Bundle?) {
        super.getBundleExtras(extras)
        extras?.let {
            mType = it.getString(ACTIVITY_COMMON_TYPE, ACTIVITY_TYPE_YZR)
            mSelectTabIndex = it.getInt(ACTIVITY_COMMON_POSITION, 0)
        }
    }

    override fun initViewsAndEvents() {
        when (mType) {
            ACTIVITY_TYPE_YZR -> initYzr()
            ACTIVITY_TYPE_LLJL -> initLljl()
            ACTIVITY_TYPE_LXJL -> initLxfs()
            ACTIVITY_TYPE_DZH -> initDzh()
            ACTIVITY_TYPE_ZNX -> initZnx()
            ACTIVITY_TYPE_TZ -> initTz()
            ACTIVITY_TYPE_XFJL -> initXfjl()
            else -> {}
        }

        initVp()
        iv_left_icon.setOnClickListener(this)
    }

    private fun initXfjl() {
        tv_title.text = "充值/消费记录"

        tabsData.add("消费记录")
        tabsData.add("充值记录")

        fragments.add(CommonListFragment.newInstance(ACTIVITY_TYPE_XFJL))
        fragments.add(CommonListFragment.newInstance(ACTIVITY_TYPE_XFJL))
    }

    private fun initTz() {
        tv_title.text = "通知"

        tabsData.add("系统通知")
        tabsData.add("评论通知")
        tabsData.add("相亲通知")

        fragments.add(CommonListFragment.newInstance(ACTIVITY_TYPE_TZ))
        fragments.add(CommonListFragment.newInstance(ACTIVITY_TYPE_TZ))
        fragments.add(CommonListFragment.newInstance(ACTIVITY_TYPE_TZ))
    }

    private fun initZnx() {
        tv_title.text = "站内信"

        tabsData.add("收件箱")
        tabsData.add("发件箱")

        fragments.add(CommonListFragment.newInstance(ACTIVITY_TYPE_ZNX))
        fragments.add(CommonListFragment.newInstance(ACTIVITY_TYPE_ZNX))
    }

    private fun initDzh() {
        tv_title.text = "打招呼"

        tabsData.add("谁跟我打招呼")
        tabsData.add("我跟谁打招呼")

        fragments.add(CommonListFragment.newInstance(ACTIVITY_TYPE_DZH))
        fragments.add(CommonListFragment.newInstance(ACTIVITY_TYPE_DZH))
    }

    private fun initLxfs() {
        tv_title.text = "联系记录"

        tabsData.add("我联系过的")
        tabsData.add("谁联系过我")

        fragments.add(CommonListFragment.newInstance(ACTIVITY_TYPE_LXJL))
        fragments.add(CommonListFragment.newInstance(ACTIVITY_TYPE_LXJL))
    }

    private fun initLljl() {
        tv_title.text = "浏览记录"

        tabsData.add("谁浏览过我")
        tabsData.add("我浏览过谁")

        fragments.add(CommonListFragment.newInstance(ACTIVITY_TYPE_LLJL))
        fragments.add(CommonListFragment.newInstance(ACTIVITY_TYPE_LLJL))
    }

    private fun initYzr() {
        tv_title.text = "意中人"

        tabsData.add("我的意中人")
        tabsData.add("谁加我为意中人")

        fragments.add(CommonListFragment.newInstance(ACTIVITY_TYPE_YZR))
        fragments.add(CommonListFragment.newInstance(ACTIVITY_TYPE_YZR))
    }

    private fun initVp() {
        vpAdapter = CommonTabAdapter(tabsData, fragments, this)
        vp2.adapter = vpAdapter

        TabLayoutMediator(tab_common, vp2) { tab, position ->
            // 初始化Tab
            tab.id = position
            if (tabsData.size > 0) {
                tab.customView = getTabView(tabsData[position])
            }
        }.attach()
        tab_common.addOnTabSelectedListener(object : TabLayout.OnTabSelectedListener {
            override fun onTabSelected(tab: TabLayout.Tab?) {
                updateTab(tab, true)
                mSelectTabIndex = tab!!.id
            }

            override fun onTabUnselected(tab: TabLayout.Tab?) {
                updateTab(tab, false)
            }

            override fun onTabReselected(tab: TabLayout.Tab?) {
            }
        })
        tab_common.getTabAt(mSelectTabIndex)?.select()
    }

    private fun getTabView(text: String): View {
        val view = LayoutInflater.from(this).inflate(R.layout.item_video_tab, null)
        view.findViewById<TextView>(R.id.item_tv).text = text
        return view
    }

    private fun updateTab(tab: TabLayout.Tab?, isSelected: Boolean) {
        tab?.customView?.let {
            val text = it.findViewById<TextView>(R.id.item_tv)
            if (isSelected) {
                text.setTextColor(ContextCompat.getColor(this, R.color.red))
            } else {
                text.setTextColor(ContextCompat.getColor(this, R.color.color_8D8F90))
            }
        }
    }

    override fun onClick(v: View?) {
        if (AntiShake.check(v?.id)) return

        when (v?.id) {
            R.id.iv_left_icon -> finish()
        }
    }
}
