package com.duben.roseplaylet.ui.activitys

import android.graphics.Color
import android.os.Bundle
import android.util.Log
import android.view.Gravity
import android.view.View
import android.view.ViewGroup
import android.widget.FrameLayout
import android.widget.TextView
import android.widget.Toast
import com.bytedance.sdk.djx.DJXRewardAdResult
import com.bytedance.sdk.djx.DJXSdk
import com.bytedance.sdk.djx.IDJXWidget
import com.bytedance.sdk.djx.interfaces.listener.IDJXAdListener
import com.bytedance.sdk.djx.interfaces.listener.IDJXDramaListener
import com.bytedance.sdk.djx.interfaces.listener.IDJXDramaUnlockListener
import com.bytedance.sdk.djx.model.*
import com.bytedance.sdk.djx.params.DJXWidgetDramaDetailParams
import com.bytedance.sdk.openadsdk.*
import com.duben.library.utils.nodoubleclick.AntiShake
import com.duben.roseplaylet.BuildConfig
import com.duben.roseplaylet.R
import com.duben.roseplaylet.common.Constant
import com.duben.roseplaylet.mvp.model.VedioBean
import com.duben.roseplaylet.ui.activitys.base.BaseActivity
import com.duben.roseplaylet.ui.widgets.UnlockDialog
import com.duben.roseplaylet.utils.LogUtil
import com.google.gson.Gson

/**
 * 描述：穿山甲短剧详情
 * 作者：孟崔广
 * 时间：2025/1/26 18:39
 */
class CsjDetailActivity : BaseActivity(), View.OnClickListener {

    companion object {
        private const val TAG = "CsjDetailActivity"
        var outerDrama: DJXDrama? = null
        var enterFrom: DJXWidgetDramaDetailParams.DJXDramaEnterFrom =
            DJXWidgetDramaDetailParams.DJXDramaEnterFrom.DEFAULT
    }

    // 看广告解锁剧集数
    private val lockSet=3

    private var dpWidget: IDJXWidget? = null
    private var isInited = false
    private var drama: DJXDrama? = null
    private var isRewardArrived = false
    private var vedioBean: VedioBean? = null

    // 广告模式
    private val adMode by lazy {
        DJXDramaUnlockAdMode.MODE_SPECIFIC
    }

    override fun getContentViewLayoutID() = R.layout.activity_csjdetail

    override fun isApplyKitKatTranslucency() = false

    override fun initViewsAndEvents() {
        drama = outerDrama

        if (DJXSdk.isStartSuccess()) {
            init()
        }
        initListener()
    }

    override fun getBundleExtras(extras: Bundle?) {
        super.getBundleExtras(extras)
        val json = extras?.getString(Constant.VEDIO_BEAN)
        vedioBean = Gson().fromJson(json, VedioBean::class.java)
    }

    override fun onClick(v: View?) {
        if (AntiShake.check(v?.id)) return

        when (v?.id) {
//            R.id.tvAboutasPolicy -> {
//                val bundle = Bundle()
//                bundle.putString(WebActivity.WEB_TITLE, getString(R.string.privacy_name))
//                bundle.putString(WebActivity.WEB_URL, Constant.PRIVACY_URL)
//                readyGo(WebActivity::class.java, bundle)
//            }
        }
    }

    private fun initListener() {
    }

    private fun init() {
        if (isInited) {
            return
        }
        initWidget()

        dpWidget?.let { widget ->
            supportFragmentManager.beginTransaction().replace(R.id.fl_container, widget.fragment)
                .commit()
        }

        isInited = true
    }

    private fun initWidget() {
        vedioBean?.let {
            val detailConfig = DJXDramaDetailConfig.obtain(adMode, it.freeIndex, object :
                IDJXDramaUnlockListener {
                override fun unlockFlowStart(
                    drama: DJXDrama,
                    callback: IDJXDramaUnlockListener.UnlockCallback,
                    map: Map<String, Any>?
                ) {
                    // 解锁支持多种方式：支付、广告。根据业务需求自行定义
                    // 如果在其他时机已经购买会员可以设置 unlockInfo 中的 hasMember 为 true
                    // val info = DJXDramaUnlockInfo(drama.id, lockSet, DJXDramaUnlockMethod.METHOD_PAY_MEMBER, true)
                    // callback.onConfirm(info)
                    if (map?.get("is_continuity_unlock") as? Boolean == true) {
//                        showToast("123123")
                        // 连续解锁
//                    showContinuesDialog(this@CsjDetailActivity, vedioBean?.unlockIndex, drama, callback, map, drama.index, map?.get("first_unlock_index") as? Int)
                    } else {
                        //demo仅演示广告解锁
                        UnlockDialog(this@CsjDetailActivity).apply {
                            setListener(
                                ad = {
                                    val unlockType = DJXUnlockModeType.UNLOCKTYPE_DEFAULT
                                    //点击激励视频解锁
                                    val info = DJXDramaUnlockInfo(
                                        drama.id,
                                        lockSet,
                                        DJXDramaUnlockMethod.METHOD_AD,
                                        false,
                                        unlockType = unlockType
                                    )
                                    callback.onConfirm(info)
                                },
                                close = {
                                    hide()
                                    val info = DJXDramaUnlockInfo(
                                        drama.id,
                                        lockSet,
                                        DJXDramaUnlockMethod.METHOD_AD,
                                        cancelUnlock = true
                                    )
                                    callback.onConfirm(info)
                                }
                            )
                            show()
                        }
                    }
                }

                override fun unlockFlowEnd(
                    drama: DJXDrama,
                    errCode: IDJXDramaUnlockListener.UnlockErrorStatus?,
                    map: Map<String, Any>?
                ) {
                    LogUtil.d(TAG, "unlockFlowEnd: $drama, code: $errCode, map: $map")
                }

                override fun showCustomAd(
                    drama: DJXDrama,
                    callback: IDJXDramaUnlockListener.CustomAdCallback
                ) {

                    val adSlot = AdSlot.Builder()
                        .setCodeId(BuildConfig.GROMORE_VIDEO_CODE) // 广告代码位Id
                        .setAdLoadType(TTAdLoadType.LOAD) // 本次广告用途：TTAdLoadType.LOAD实时；TTAdLoadType.PRELOAD预请求
                        .build()

                    TTAdSdk.getAdManager().createAdNative(this@CsjDetailActivity)
                        .loadRewardVideoAd(adSlot, object : TTAdNative.RewardVideoAdListener {
                            override fun onError(p0: Int, p1: String?) {
                                callback.onError()
                            }

                            override fun onRewardVideoAdLoad(ad: TTRewardVideoAd?) {
                                ad?.apply {
                                    setRewardAdInteractionListener(object :
                                        TTRewardVideoAd.RewardAdInteractionListener {
                                        override fun onAdShow() {
                                            Toast.makeText(
                                                this@CsjDetailActivity,
                                                "自定义广告展示",
                                                Toast.LENGTH_LONG
                                            ).show()
                                            callback.onShow("") // CSJ cpm 不对外，可以参考 GroMore getShowEcpm 方法获取
                                        }

                                        override fun onAdVideoBarClick() {
                                            // 广告点击
                                        }

                                        override fun onAdClose() {
                                            // 广告关闭
                                        }

                                        override fun onVideoComplete() {
                                            // 广告素材播放完成，例如视频未跳过，完整的播放了
                                        }

                                        override fun onVideoError() {
                                            // 广告展示时出错
                                            callback.onRewardVerify(DJXRewardAdResult(false))
                                        }

                                        override fun onRewardVerify(
                                            rewardVerify: Boolean,
                                            rewardAmount: Int,
                                            rewardName: String,
                                            errorCode: Int,
                                            errorMsg: String
                                        ) {
                                            // 已废弃 请使用 onRewardArrived 替代
                                        }

                                        override fun onRewardArrived(
                                            isRewardValid: Boolean,
                                            rewardType: Int,
                                            extraInfo: Bundle
                                        ) {
                                            val result = DJXRewardAdResult(isRewardValid)
                                            isRewardArrived = isRewardValid
                                            callback.onRewardVerify(result)
                                        }

                                        override fun onSkippedVideo() {
                                            // 用户在观看时点击了跳过
                                            if (!isRewardArrived) {
                                                callback.onRewardVerify(DJXRewardAdResult(false))
                                            }
                                        }
                                    })
                                    showRewardVideoAd(this@CsjDetailActivity)
                                }
                            }

                            override fun onRewardVideoCached() {}

                            override fun onRewardVideoCached(p0: TTRewardVideoAd?) {}

                        })
                }
            }).apply {
                infiniteScrollEnabled(true)
                listener(dramaDetailListener)
                adListener(object : IDJXAdListener() {

                })
//                hideLikeButton(intent.getBooleanExtra(DramaDetailConfigActivity.KEY_DRAMA_HIDE_LIKE_BUTTON, false))
//                hideFavorButton(intent.getBooleanExtra(DramaDetailConfigActivity.KEY_DRAMA_HIDE_FAVOR_BUTTON, false))
//                hideRewardDialog(intent.getBooleanExtra(DramaDetailConfigActivity.KEY_DRAMA_HIDE_REWARD_DIALOG, false))
//                hideBack(intent.getBooleanExtra(DramaDetailConfigActivity.KEY_DRAMA_HIDE_BACK, false), null)
//                hideTopInfo(intent.getBooleanExtra(DramaDetailConfigActivity.KEY_DRAMA_HIDE_TOP_INFO, false))
//                hideBottomInfo(intent.getBooleanExtra(DramaDetailConfigActivity.KEY_DRAMA_HIDE_BOTTOM_INFO, false))
//                hideMore(intent.getBooleanExtra(DramaDetailConfigActivity.KEY_DRAMA_HIDE_MORE, false))
//                hideCellularToast(intent.getBooleanExtra(DramaDetailConfigActivity.KEY_DRAMA_HIDE_CELLULAR_TOAST, false))
//                scriptTipsTopMargin = intent.getIntExtra(DramaDetailConfigActivity.KEY_DRAMA_SCRIPT_TOP_MARGIN, -1)
//                icpTipsBottomMargin = intent.getIntExtra(DramaDetailConfigActivity.KEY_DRAMA_ICP_BOTTOM_MARGIN, -1)
//                topOffset = intent.getIntExtra(DramaDetailConfigActivity.KEY_DRAMA_TOP_OFFSET, -1)
//                bottomOffset = intent.getIntExtra(DramaDetailConfigActivity.KEY_DRAMA_BOTTOM_OFFSET, -1)
//
//                if (enableCustomReport) {
//                    setCustomReport { context, id ->
//                        enterCustomReport(context, id)
//                    }
//                }

//            if (insertDrawAd) {
//                adCustomProvider(object : IDJXDramaAdCustomProvider() {
//                    override fun getDetailDrawAdPositions(): List<Int>? {
//                        return insertDrawAdConfigStr?.toNumberList()
//                    }
//
//                    override fun getDetailAdDrawView(): IDJXCustomView {
//                        return DramaDrawAdView(this@CsjDetailActivity)
//                    }
//                })
//            }
            }

            drama?.let { drama ->
                dpWidget = DJXSdk.factory().createDramaDetail(
                    DJXWidgetDramaDetailParams.obtain(drama.id, drama.index, detailConfig)
                        .currentDuration(0)
                        .fromGid(vedioBean!!.groupId.toString()) // 必传，否则影响推荐效果
                        .from(enterFrom) // 必传，否则影响推荐效果
                )
            }
        }
    }

//    private val dramaAdListener: IDJXAdListener by lazy {
//
//    }

    private val dramaDetailListener: IDJXDramaListener by lazy {
        object : IDJXDramaListener() {
            override fun createCustomView(
                container: ViewGroup?,
                map: MutableMap<String, Any>?
            ): View? {
                return if (false) {
                    container ?: return null
                    map ?: return null

                    Log.d(TAG, "createCustomView: map=$map")
                    val label = TextView(container.context)
                    label.text = map["title"]?.toString() ?: ""
                    label.setTextColor(Color.parseColor("#f1f1f1"))
                    label.textSize = 20f

                    val labelParams = FrameLayout.LayoutParams(
                        FrameLayout.LayoutParams.WRAP_CONTENT,
                        FrameLayout.LayoutParams.WRAP_CONTENT
                    )
                    labelParams.topMargin = 300
                    labelParams.gravity = Gravity.CENTER_HORIZONTAL
                    label.layoutParams = labelParams

                    val customView = FrameLayout(container.context)
                    customView.layoutParams = ViewGroup.LayoutParams(
                        ViewGroup.LayoutParams.MATCH_PARENT,
                        ViewGroup.LayoutParams.MATCH_PARENT
                    )
                    customView.addView(label)

                    return customView
                } else {
                    super.createCustomView(container, map)
                }
            }
        }
    }
}
