package com.duben.roseplaylet.ui.activitys

import android.content.Context
import android.content.Intent
import android.media.AudioManager
import android.net.Uri
import android.os.Bundle
import android.view.KeyEvent
import android.view.View
import androidx.fragment.app.Fragment
import androidx.fragment.app.FragmentTransaction
import com.duben.roseplaylet.R
import com.duben.roseplaylet.common.AppConfig
import com.duben.roseplaylet.common.Constant
import com.duben.roseplaylet.ui.activitys.base.BaseActivity
import com.duben.roseplaylet.ui.fragment.*
import com.duben.roseplaylet.ui.widgets.DialogListener
import com.duben.roseplaylet.ui.widgets.PhoneDialog
import kotlinx.android.synthetic.main.activity_main.*

/**
 * 描述：首页
 * 作者：孟崔广
 * 时间：2022/12/24 18:06
 */
class MainActivity : BaseActivity(), View.OnClickListener {

    private var audioManager: AudioManager? = null

    // 底部标签切换的Fragment
    private var videoFragment: Fragment? = null
    private var squareFragment: Fragment? = null
    private var inviteFragment: Fragment? = null
    private var msgFragment: Fragment? = null
    private var myFragment: Fragment? = null

    private var currentFragment: Fragment? = null
    private var phoneDialog: PhoneDialog? = null

    override fun getContentViewLayoutID(): Int {
        return R.layout.activity_main
    }

    override fun initViewsAndEvents() {
        audioManager = getSystemService(Context.AUDIO_SERVICE) as AudioManager

        tab_rl_video.setOnClickListener(this)
        tab_rl_square.setOnClickListener(this)
        tab_rl_invite.setOnClickListener(this)
        tab_rl_msg.setOnClickListener(this)
        tab_rl_my.setOnClickListener(this)

        AppConfig.fragmentClickFlag = Constant.FRAGMENT_CLICK_ONE
        if (videoFragment == null) {
            videoFragment = FirstFragment()
        }
        if (!videoFragment!!.isAdded) {
            // 提交事务
            supportFragmentManager.beginTransaction()
                .add(R.id.content_layout, videoFragment!!).commitAllowingStateLoss()
            // 记录当前Fragment
            currentFragment = videoFragment
        }
        tab_iv_video.isSelected = true
        tab_tv_video.isSelected = true
    }

    override fun isApplyKitKatTranslucency(): Boolean {
        return false
    }

    override fun onDestroy() {
        super.onDestroy()
        currentFragment = null
        audioManager = null
    }

    public override fun onSaveInstanceState(outState: Bundle) {
        //将这一行注释掉，阻止activity保存fragment的状态，防止UI重叠
        //super.onSaveInstanceState(outState);
    }

    var oldTime: Long = 0
    override fun onKeyDown(keyCode: Int, event: KeyEvent): Boolean {
        when (keyCode) {
            KeyEvent.KEYCODE_VOLUME_UP -> {
                audioManager?.adjustStreamVolume(
                    AudioManager.STREAM_MUSIC,
                    AudioManager.ADJUST_RAISE,
                    AudioManager.FX_FOCUS_NAVIGATION_UP
                )
                return true
            }
            KeyEvent.KEYCODE_VOLUME_DOWN -> {
                audioManager?.adjustStreamVolume(
                    AudioManager.STREAM_MUSIC,
                    AudioManager.ADJUST_LOWER,
                    AudioManager.FX_FOCUS_NAVIGATION_UP
                )
                return true
            }
            KeyEvent.KEYCODE_BACK -> {
                // 设置为后台
                val currentTime = System.currentTimeMillis()
                if (currentTime - oldTime < 2 * 1000) {
                    baseApplication.exitApp()
                } else {
                    showToast("再次点击退出" + getString(R.string.app_name))
                    oldTime = currentTime
                }
            }
        }
        return true
    }


    override fun onClick(view: View) {
        when (view.id) {
            R.id.tab_rl_video -> clickTab1Layout()
            R.id.tab_rl_square -> clickTab2Layout()
            R.id.tab_rl_invite -> clickTab3Layout()
            R.id.tab_rl_msg -> clickTab4Layout()
            R.id.tab_rl_my -> clickTab5Layout()
        }
    }

    /**
     * 点击第一个tab
     */
    fun clickTab1Layout() {
        AppConfig.fragmentClickFlag = Constant.FRAGMENT_CLICK_ONE
        if (videoFragment == null) {
            videoFragment = FirstFragment()
        }
        addOrShowFragment(supportFragmentManager.beginTransaction(), videoFragment!!)
        tab_iv_video.isSelected = true
        tab_tv_video.isSelected = true
        tab_iv_square.isSelected = false
        tab_tv_square.isSelected = false
        tab_iv_invite.isSelected = false
        tab_tv_invite.isSelected = false
        tab_iv_msg.isSelected = false
        tab_tv_msg.isSelected = false
        tab_iv_my.isSelected = false
        tab_tv_my.isSelected = false
    }

    /**
     * 点击第二个tab
     */
    fun clickTab2Layout() {
        AppConfig.fragmentClickFlag = Constant.FRAGMENT_CLICK_TWO
        if (squareFragment == null) {
            squareFragment = SquareFragment()
        }
        addOrShowFragment(supportFragmentManager.beginTransaction(), squareFragment!!)
        tab_iv_video.isSelected = false
        tab_tv_video.isSelected = false
        tab_iv_square.isSelected = true
        tab_tv_square.isSelected = true
        tab_iv_invite.isSelected = false
        tab_tv_invite.isSelected = false
        tab_iv_msg.isSelected = false
        tab_tv_msg.isSelected = false
        tab_iv_my.isSelected = false
        tab_tv_my.isSelected = false
    }

    /**
     * 点击第三个tab
     */
    fun clickTab3Layout() {
        AppConfig.fragmentClickFlag = Constant.FRAGMENT_CLICK_THREE
        if (inviteFragment == null) {
            inviteFragment = InviteFragment()
        }
        addOrShowFragment(supportFragmentManager.beginTransaction(), inviteFragment!!)
        tab_iv_video.isSelected = false
        tab_tv_video.isSelected = false
        tab_iv_square.isSelected = false
        tab_tv_square.isSelected = false
        tab_iv_invite.isSelected = true
        tab_tv_invite.isSelected = true
        tab_iv_msg.isSelected = false
        tab_tv_msg.isSelected = false
        tab_iv_my.isSelected = false
        tab_tv_my.isSelected = false
    }

    /**
     * 点击第四个tab
     */
    fun clickTab4Layout() {
        AppConfig.fragmentClickFlag = Constant.FRAGMENT_CLICK_FOUR
        if (msgFragment == null) {
            msgFragment = MessageFragment()
        }
        addOrShowFragment(supportFragmentManager.beginTransaction(), msgFragment!!)
        tab_iv_video.isSelected = false
        tab_tv_video.isSelected = false
        tab_iv_square.isSelected = false
        tab_tv_square.isSelected = false
        tab_iv_invite.isSelected = false
        tab_tv_invite.isSelected = false
        tab_iv_msg.isSelected = true
        tab_tv_msg.isSelected = true
        tab_iv_my.isSelected = false
        tab_tv_my.isSelected = false
    }

    /**
     * 点击第五个tab
     */
    fun clickTab5Layout() {
        AppConfig.fragmentClickFlag = Constant.FRAGMENT_CLICK_FIVE
        if (myFragment == null) {
            myFragment = MyFragment()
        }
        addOrShowFragment(supportFragmentManager.beginTransaction(), myFragment!!)
        tab_iv_video.isSelected = false
        tab_tv_video.isSelected = false
        tab_iv_square.isSelected = false
        tab_tv_square.isSelected = false
        tab_iv_invite.isSelected = false
        tab_tv_invite.isSelected = false
        tab_iv_msg.isSelected = false
        tab_tv_msg.isSelected = false
        tab_iv_my.isSelected = true
        tab_tv_my.isSelected = true
    }

    /**
     * 添加或者显示碎片
     *
     * @param transaction
     * @param fragment
     */
    private fun addOrShowFragment(
        transaction: FragmentTransaction,
        fragment: Fragment
    ) {
        if (currentFragment === fragment) return
        if (!fragment.isAdded) { // 如果当前fragment未被添加，则添加到Fragment管理器中
            transaction.hide(currentFragment!!)
                .add(R.id.content_layout, fragment).commitAllowingStateLoss()
        } else {
            transaction.hide(currentFragment!!).show(fragment).commitAllowingStateLoss()
        }
        currentFragment = fragment
    }

    fun backPhoneDialog() {
        phoneDialog = PhoneDialog(context, object : DialogListener() {
            override fun onClick(v: View) {
                if (!isFinishing && phoneDialog != null && phoneDialog!!.isShowing) {
                    phoneDialog!!.dismiss()
                }
                when (v.id) {
                    R.id.tv_phone_dialog_phone, R.id.tv_phone_dialog_phone2 -> {
                        val intent = Intent(Intent.ACTION_DIAL)
                        intent.data = Uri.parse("tel:" + "4000969950")
                        context.startActivity(intent)
                    }
                }
            }
        })
        phoneDialog!!.show()
    }

}