package com.duben.roseplaylet.ui.activitys

import android.Manifest
import android.annotation.SuppressLint
import android.app.Activity
import android.content.Intent
import android.content.pm.ActivityInfo
import android.text.TextUtils
import android.view.View
import com.luck.picture.lib.PictureSelector
import com.luck.picture.lib.compress.Luban
import com.luck.picture.lib.config.PictureConfig
import com.luck.picture.lib.config.PictureMimeType
import com.luck.picture.lib.entity.LocalMedia
import com.luck.picture.lib.style.PictureWindowAnimationStyle
import com.luck.picture.lib.thread.PictureThreadUtils
import com.luck.picture.lib.tools.DateUtils
import com.luck.picture.lib.tools.SdkVersionUtils
import com.duben.library.net.neterror.BaseSubscriber
import com.duben.roseplaylet.R
import com.duben.roseplaylet.manager.AppHttpManager
import com.duben.roseplaylet.manager.OssManager
import com.duben.roseplaylet.mvp.model.BaseResponse
import com.duben.roseplaylet.mvp.model.OrderRecordBean
import com.duben.roseplaylet.ui.activitys.base.BaseActivity
import com.duben.roseplaylet.ui.widgets.nineImage.AddImagesInfo
import com.duben.roseplaylet.ui.widgets.nineImage.AddImagesView
import com.duben.roseplaylet.utils.GlideEngine
import com.duben.roseplaylet.utils.SpanUtils
import com.tbruyelle.rxpermissions.RxPermissions
import com.yalantis.ucrop.UCrop
import kotlinx.android.synthetic.main.activity_template.*
import kotlinx.android.synthetic.main.header_layout.*
import java.io.File
import java.util.*
import kotlin.collections.HashMap

/**
 * 描述：申请模板
 */
@SuppressLint("SetTextI18n")
class TemplateActivity : BaseActivity(), View.OnClickListener {

    private var mOssList = arrayListOf<String>()

    // 图片选择框架跳转动画
    private lateinit var mWindowAnimationStyle: PictureWindowAnimationStyle

    override fun getContentViewLayoutID() = R.layout.activity_template

    override fun isApplyKitKatTranslucency() = false

    override fun initViewsAndEvents() {
        tv_title.text = "计数模型申请"
        iv_left_icon.visibility = View.VISIBLE
        iv_left_icon.setImageResource(R.mipmap.ic_arrow_back)

        tv_name.text = SpanUtils()
            .append("*")
            .setForegroundColor(resources.getColor(R.color.color_FF4144))
            .append("需要计数的物品名称")
            .setForegroundColor(resources.getColor(R.color.black))
            .create()

        tv_phone.text = SpanUtils()
            .append("*")
            .setForegroundColor(resources.getColor(R.color.color_FF4144))
            .append("联系方式")
            .setForegroundColor(resources.getColor(R.color.black))
            .create()

        tv_photo.text = SpanUtils()
            .append("*")
            .setForegroundColor(resources.getColor(R.color.color_FF4144))
            .append("需要计数的物品照片（5-10张图片）")
            .setForegroundColor(resources.getColor(R.color.black))
            .create()

        mWindowAnimationStyle = PictureWindowAnimationStyle()
        mWindowAnimationStyle.ofAllAnimation(
            com.luck.picture.lib.R.anim.picture_anim_up_in,
            com.luck.picture.lib.R.anim.picture_anim_down_out
        )

        initListener()
    }


    override fun onClick(v: View?) {
        when (v?.id) {
            R.id.iv_left_icon -> finish()
            R.id.btn_submit -> {
                if (TextUtils.isEmpty(et_name.text)) {
                    showToast("申请名称不能为空")
                    return
                }
                if (TextUtils.isEmpty(et_phone.text)) {
                    showToast("联系方式不能为空")
                    return
                }
                if (aiv.getAddImages().size < 5) {
                    showToast("模板照片最少需要5张")
                    return
                }
                mOssList.clear()
                luban()
            }
        }
    }

    private fun initListener() {
        iv_left_icon.setOnClickListener(this)
        btn_submit.setOnClickListener(this)
        aiv.setAddImagesViewListener(object : AddImagesView.AddImagesViewListener {
            override fun onAdd() {
                showAlbum()
            }

            override fun onDelete(position: Int) {
            }

            override fun onPreview(item: AddImagesInfo?, lists: List<AddImagesInfo>?) {

            }
        })
    }

    private fun showAlbum() {
        RxPermissions(this)
            .request(Manifest.permission.WRITE_EXTERNAL_STORAGE)
            .subscribe { granted: Boolean ->
                if (granted) {
                    val isEnableCrop = false
                    PictureSelector.create(this)
                        .openGallery(PictureMimeType.ofImage())
                        .theme(R.style.picture_WeChat_style) // 微信主题色
                        .isWeChatStyle(true) // 微信样式
                        .isCamera(false) //列表是否显示拍照按钮
                        .isZoomAnim(isEnableCrop)//图片选择缩放效果
                        .isEnableCrop(isEnableCrop)//是否开启裁剪
                        .setPictureWindowAnimationStyle(mWindowAnimationStyle)
                        .withAspectRatio(2, 4)
                        .isCompress(true)//是否压缩
                        .compressFocusAlpha(false)//压缩后是否保持图片的透明通道
                        .minimumCompressSize(200)// 小于多少kb的图片不压缩
                        .compressQuality(60)//图片压缩后输出质量
                        .synOrAsy(true)//开启同步or异步压缩
                        .maxSelectNum(10)
                        .setRequestedOrientation(ActivityInfo.SCREEN_ORIENTATION_PORTRAIT)//屏幕旋转方向
                        .selectionMode(PictureConfig.MULTIPLE)//单选or多选
                        .imageEngine(GlideEngine.createGlideEngine())// 图片加载引擎
                        .forResult(PictureConfig.CHOOSE_REQUEST)

                } else {
                    showMissingPermissionDialog("存储")
                }
            }
    }

    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?) {
        super.onActivityResult(requestCode, resultCode, data)
        if (resultCode == Activity.RESULT_OK) {
            when (requestCode) {
                // 相册获取图片
                PictureConfig.CHOOSE_REQUEST -> {
                    val selectList: List<LocalMedia> = PictureSelector.obtainMultipleResult(data)

                    val imgList = arrayListOf<AddImagesInfo>()
                    for (localMedia in selectList) {
                        val addImagesInfo = AddImagesInfo(localMedia.compressPath, "2")
                        imgList.add(addImagesInfo)
                    }
                    aiv.addAddImages(imgList)
                }
            }
        } else if (resultCode == UCrop.RESULT_ERROR) {
            showToast("裁剪失败，请再试一下")
        }
    }


    /**
     * 鲁班压缩
     */
    private fun luban() {
        val dataList = arrayListOf<LocalMedia>()
        aiv.getAddImages().let {
            for (addImagesInfo in it) {
                val cutPath = addImagesInfo.imgUrl as String
                val media = LocalMedia(cutPath, 0, false, 1, 0, PictureMimeType.ofImage())
                media.cutPath = cutPath
                if (SdkVersionUtils.checkedAndroid_Q()) {
                    media.androidQToPath = cutPath
                }
                dataList.add(media)
            }
        }


        runOnUiThread {
            showLoading("加载中...")
        }

        val newFileName = DateUtils.getCreateFileName("IMG_CMP_") + ".jpeg"

        PictureThreadUtils.executeByIo(object : PictureThreadUtils.SimpleTask<List<File>>() {
            @Throws(Exception::class)
            override fun doInBackground(): List<File> {
                return Luban.with(mContext)
                    .loadMediaData<LocalMedia>(dataList)
                    .ignoreBy(200)
                    .isCamera(false)
                    .setCompressQuality(80)
                    .setTargetDir(null)
                    .setFocusAlpha(false)
                    .setNewCompressFileName(newFileName).get()
            }

            override fun onSuccess(files: List<File>) {
                for (file in files) {
                    uploadOss(file.path, "APPLY_TEMPLATE")
                }
            }

            override fun onFail(t: Throwable?) {
                super.onFail(t)
                hideLoading()
                showToast("网络出小差了！")
            }
        })
    }

    /**
     * 上传到oss服务器
     */
    private fun uploadOss(localFile: String, type: String) {
        OssManager.instance?.run {
            uploadImg(type, localFile)
            setOssUploadImgListener(object : OssManager.OssUploadImgListener {
                override fun uploadImgFailForSubthread(errorInfo: String) {
                    runOnUiThread {
                        showToast(errorInfo)
                    }
                }

                override fun uploadImgSuccessForSubthread(imgPath: String?) {
                    runOnUiThread {
                        if (TextUtils.isEmpty(imgPath)) {
                            showToast("图片异常，请重新添加")
                            return@runOnUiThread
                        }
                        mOssList.add(imgPath!!)

                        if (mOssList.size == aiv.getAddImages().size) {

                            val hashMap = HashMap<String, Any>()
                            hashMap["title"] = et_name.text.toString()
                            hashMap["mobile"] = et_phone.text.toString()
                            hashMap["imgs"] = mOssList.toArray()
//                            AppHttpManager.getInstance(baseApplication)
//                                .call(baseApplication.loanService.numFeedback(hashMap),
//                                    object : BaseSubscriber<BaseResponse<OrderRecordBean>>() {
//                                        override fun onError(e: Throwable) {
//                                            hideLoading()
//                                        }
//
//                                        override fun onNext(t: BaseResponse<OrderRecordBean>) {
//                                            hideLoading()
//                                            finish()
//                                        }
//                                    })
                        }
                    }
                }
            })
        }
    }
}
