package com.duben.roseplaylet.ui.activitys

import android.content.Intent
import android.graphics.Color
import android.view.View
import com.duben.library.utils.nodoubleclick.AntiShake
import com.duben.roseplaylet.R
import com.duben.roseplaylet.common.UserProfile
import com.duben.roseplaylet.ui.activitys.base.BaseActivity
import com.duben.roseplaylet.ui.adapter.FlowTagAdapter
import kotlinx.android.synthetic.main.activity_user_label.*
import kotlinx.android.synthetic.main.header_layout.*

class UserLabelActivity : BaseActivity(), View.OnClickListener {

    companion object {
        const val MAX_SELECT_COUNT = 6
    }

    private var selectCount = 0

    private val usualData = arrayListOf<String>()
    private val specialData = arrayListOf<String>()

    override fun getContentViewLayoutID() = R.layout.activity_user_label

    override fun isApplyKitKatTranslucency() = false

    override fun initViewsAndEvents() {
        tv_right.visibility = View.VISIBLE
        tv_title.text = "个性标签"
        tv_right.text = "确定"
        tv_title.setTextColor(Color.WHITE)
        tv_right.setTextColor(Color.WHITE)
        iv_left_icon.visibility = View.VISIBLE
        iv_left_icon.setImageResource(R.mipmap.ic_arrow_white)

        tv_label1.text = UserProfile.USUAL_LABEL.str
        tv_label2.text = UserProfile.SPECIAL_LABEL.str

        usualData.addAll(UserProfile.USUAL_LABEL.arr)
        specialData.addAll(UserProfile.SPECIAL_LABEL.arr)

        val usualAdapter = FlowTagAdapter(this, usualData)
        flowlayout_usual.adapter = usualAdapter

        val specialAdapter = FlowTagAdapter(this, specialData)
        flowlayout_special.adapter = specialAdapter

        flowlayout_usual.setOnTagClickListener { view, position, parent ->
            updateData()
            false
        }

        flowlayout_special.setOnTagClickListener { view, position, parent ->
            updateData()
            false
        }

        initListener()
    }

    private fun updateData() {
        selectCount = flowlayout_usual.selectedList.size + flowlayout_special.selectedList.size

        if (MAX_SELECT_COUNT - flowlayout_usual.selectedList.size <= 0) {
            flowlayout_usual.setMaxSelectCount(MAX_SELECT_COUNT)
            fl_special.setIntercepting(true)
        } else if (MAX_SELECT_COUNT - flowlayout_special.selectedList.size <= 0) {
            flowlayout_special.setMaxSelectCount(MAX_SELECT_COUNT)
            fl_usual.setIntercepting(true)
        } else if ((MAX_SELECT_COUNT - flowlayout_usual.selectedList.size - flowlayout_special.selectedList.size) <= 0) {
            fl_usual.setIntercepting(false)
            fl_special.setIntercepting(false)
            flowlayout_usual.setMaxSelectCount(flowlayout_usual.selectedList.size)
            flowlayout_special.setMaxSelectCount(flowlayout_special.selectedList.size)
        } else {
            fl_usual.setIntercepting(false)
            fl_special.setIntercepting(false)

            if (flowlayout_usual.selectedList.size > 0 && flowlayout_special.selectedList.size > 0) {
                flowlayout_usual.setMaxSelectCount(MAX_SELECT_COUNT - flowlayout_special.selectedList.size)
                flowlayout_special.setMaxSelectCount(MAX_SELECT_COUNT - flowlayout_usual.selectedList.size)
            } else if (flowlayout_usual.selectedList.size > 0 && flowlayout_special.selectedList.size <= 0) {
                flowlayout_usual.setMaxSelectCount(MAX_SELECT_COUNT)
                flowlayout_special.setMaxSelectCount(MAX_SELECT_COUNT - flowlayout_usual.selectedList.size)
            } else if (flowlayout_special.selectedList.size > 0 && flowlayout_usual.selectedList.size <= 0) {
                flowlayout_special.setMaxSelectCount(MAX_SELECT_COUNT)
                flowlayout_usual.setMaxSelectCount(MAX_SELECT_COUNT - flowlayout_special.selectedList.size)
            }
        }
        tv_select.text = "已选择${selectCount}/${MAX_SELECT_COUNT}"
    }

    override fun onClick(v: View?) {
        if (AntiShake.check(v?.id)) return

        when (v?.id) {
            R.id.iv_left_icon -> finish()
            R.id.tv_right -> {
                val labelData = arrayListOf<String>()

                for (i in 0 until usualData.size) {
                    for (j in flowlayout_usual.selectedList) {
                        if (i == j) {
                            labelData.add(usualData[i])
                        }
                    }
                }
                for (i in 0 until specialData.size) {
                    for (j in flowlayout_special.selectedList) {
                        if (i == j) {
                            labelData.add(specialData[i])
                        }
                    }
                }

                val resultIntent = Intent()
                resultIntent.putExtra(RealAuthActivity.USER_LABEL_DATA, labelData)
                setResult(RealAuthActivity.USER_LABEL_REQUEST_CODE, resultIntent)
            }
        }
    }

    private fun initListener() {
        iv_left_icon.setOnClickListener(this)
        tv_right.setOnClickListener(this)
    }
}
