package com.duben.roseplaylet.ui.adapter

import android.app.Activity
import android.content.Context
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.FrameLayout
import android.widget.TextView
import androidx.recyclerview.widget.RecyclerView
import com.bigkoo.pickerview.builder.OptionsPickerBuilder
import com.bigkoo.pickerview.listener.OnOptionsSelectChangeListener
import com.bigkoo.pickerview.view.OptionsPickerView
import com.duben.roseplaylet.R
import com.duben.roseplaylet.mvp.model.CosumerAddressEntity
import com.duben.roseplaylet.mvp.model.MainMultiItemEntity
import com.duben.roseplaylet.mvp.model.ProvinceCosumer
import com.duben.roseplaylet.utils.AddressData
import com.duben.roseplaylet.utils.ToastUtil
import java.util.*

class MenuAdapter(private val act: Activity, private val dataBean: MainMultiItemEntity) :
    RecyclerView.Adapter<RecyclerView.ViewHolder>() {
    companion object {
        const val TYPE_CITY = 0
        const val TYPE_NONE = 1
    }

    lateinit var context: Context
    lateinit var mOnItemClickListener: OnItemClickListener
    private var checkItemPosition = 0
    private var cityHolder: CityHolder? = null
    private var localCosumerAddress: CosumerAddressEntity? = null
    private var mOptions1 = 0
    private var mOptions2 = 0

    private val optionsPickerView: OptionsPickerView<Any>?
        get() {
            val pvCustomOptions = OptionsPickerBuilder(
                act
            ) { options1, option2, options3, v ->
            }
                .setLayoutRes(
                    R.layout.pickerview_custom_options
                ) { v ->
                    v.findViewById<View>(R.id.tv_add).setOnClickListener {
                        localCosumerAddress?.let {
                            val province = it.options1Items!!.get(mOptions1).name
                            val city = it.options2Items!!.get(mOptions1).get(mOptions2)
//                            ToastUtil.show(context, " options1=${province} options2=${city}")
                            mOnItemClickListener.onItemClick("${city}", 0)
                        }
                    }
                    v.findViewById<View>(R.id.tv_cancel).setOnClickListener {
                        mOnItemClickListener.onItemClick("城市不限", 0)
                    }

                }
                .setDecorView(cityHolder?.frameRoot) //非dialog模式下,设置ViewGroup, pickerView将会添加到这个ViewGroup中
                .setOutSideColor(0x00000000)
//                .setTextColorCenter(R.color.main_mints)
                .setOutSideCancelable(false)
                .setSelectOptions(mOptions1, mOptions2)
                .setOptionsSelectChangeListener(object : OnOptionsSelectChangeListener {
                    override fun onOptionsSelectChanged(
                        options1: Int,
                        options2: Int,
                        options3: Int
                    ) {
                        mOptions1 = options1
                        mOptions2 = options2

                    }

                })
                .build<Any>()
            return pvCustomOptions
        }


    override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): RecyclerView.ViewHolder {
        context = parent.context

        if (viewType == TYPE_CITY) {
            localCosumerAddress = AddressData.getLocalCosumerAddress()
            return CityHolder(
                LayoutInflater.from(parent.context).inflate(R.layout.item_city, parent, false)
            )
        } else {
            return ViewHolder(
                LayoutInflater.from(parent.context).inflate(R.layout.item_menu, parent, false)
            )
        }
    }

    override fun getItemViewType(position: Int): Int {
        return dataBean.itemType
    }

    override fun onBindViewHolder(viewHolder: RecyclerView.ViewHolder, position: Int) {
        if (viewHolder is CityHolder) {
            cityHolder = viewHolder as CityHolder
            val pvCustomOptions = optionsPickerView
            pvCustomOptions?.setPicker(
                localCosumerAddress?.options1Items as List<ProvinceCosumer>,
                localCosumerAddress?.options2Items as List<MutableList<String>>
            )
            pvCustomOptions?.show()
        } else {
            val holder = viewHolder as ViewHolder
            val dataList = dataBean.beans

            holder.tvMenu.text = (dataList!!.get(position))
            holder.itemView.setOnClickListener {
                mOnItemClickListener.onItemClick(dataList!![position], position)
            }
            if (checkItemPosition == position) {
                holder.tvMenu.setTextColor(context.getResources().getColor(R.color.color_ff33b5e5));
            } else {
                holder.tvMenu.setTextColor(context.getResources().getColor(R.color.black));
            }
        }
    }

    override fun getItemCount(): Int {
        return if (dataBean.itemType == TYPE_CITY) 1 else dataBean.beans?.size!!
    }

    inner class ViewHolder(view: View) : RecyclerView.ViewHolder(view) {
        val tvMenu: TextView = view.findViewById(R.id.tv_menu)
    }

    inner class CityHolder(view: View) : RecyclerView.ViewHolder(view) {
        val frameRoot: FrameLayout = view.findViewById(R.id.frame_root)
    }

    interface OnItemClickListener {
        fun onItemClick(title: String, position: Int)
    }

    fun setOnItemClickListener(listener: OnItemClickListener) {
        mOnItemClickListener = listener
    }

    fun setCheckItem(position: Int) {
        checkItemPosition = position
        notifyDataSetChanged()
    }

}