package com.duben.roseplaylet.ui.adapter

import android.content.Context
import android.graphics.Color
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.FrameLayout
import android.widget.TextView
import androidx.recyclerview.widget.RecyclerView
import com.duben.roseplaylet.R
import com.duben.roseplaylet.mvp.model.VipBean
import com.duben.roseplaylet.utils.BubbleUtils
import com.duben.roseplaylet.utils.SpanUtils
import com.duben.roseplaylet.utils.UIUtils

class VipAdapter(private val vipList: MutableList<VipBean.ListBean>) :
    RecyclerView.Adapter<VipAdapter.ViewHolder>() {

    var myPos: Int = 0
    lateinit var context: Context

    lateinit var mOnItemClickListener: OnItemClickListener

    init {
        if (vipList.size > 0) {
            for (i in 0 until vipList.size) {
                if (vipList[i].activityType == 1) {
                    myPos = i
                    break
                }
            }
        }
    }

    inner class ViewHolder(view: View) : RecyclerView.ViewHolder(view) {
        val rlVipItemRoot: FrameLayout = view.findViewById(R.id.rlVipItemRoot)
        val tvVipItemCurMoney: TextView = view.findViewById(R.id.tvVipItemCurMoney)
        val tvVipItemOldMoney: TextView = view.findViewById(R.id.tvVipItemOldMoney)
        val tvVipItemMoney: TextView = view.findViewById(R.id.tvVipItemMoney)
        val tvVipItemTitle: TextView = view.findViewById(R.id.tvVipItemTitle)
        val tvVipItemLable: TextView = view.findViewById(R.id.tvVipItemLable)
    }

    override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): ViewHolder {
        context = parent.context
        val view = LayoutInflater.from(context).inflate(R.layout.item_rv_vip, parent, false)
        return ViewHolder(view)
    }

    override fun onBindViewHolder(holder: ViewHolder, position: Int) {
        val vipBean = vipList[position]

        if (vipBean.trial != 0) {
            holder.tvVipItemLable.text = "限时特惠"
        } else {
            holder.tvVipItemLable.text = "热门"
        }

        holder.tvVipItemTitle.text = vipBean.title

        holder.tvVipItemOldMoney.text = SpanUtils()
            .append("原价 ")
            .setFontSize(BubbleUtils.sp2px(10))
            .setStrikethrough()
            .append("" + vipBean.oldPrice)
            .setFontSize(BubbleUtils.sp2px(12))
            .setBold()
            .setStrikethrough()
            .create()

        if (vipBean.firstPayPriceAfterBack == 0.0) {
            holder.tvVipItemCurMoney.text = SpanUtils()
                .append("￥")
                .append("0")
                .setFontSize(BubbleUtils.sp2px(36))
                .create()
        } else {
            holder.tvVipItemCurMoney.text = SpanUtils()
                .append("￥")
                .append(vipBean.firstPayPriceAfterBack.toString())
                .setFontSize(BubbleUtils.sp2px(36))
                .create()
        }

        holder.tvVipItemMoney.text = vipBean.topTitle

        if (position == 0) {
            val layoutParams = holder.rlVipItemRoot.layoutParams as FrameLayout.LayoutParams
            layoutParams.width = UIUtils.dp2px(170)
            holder.rlVipItemRoot.layoutParams = layoutParams
        }

        holder.rlVipItemRoot.setOnClickListener {
            mOnItemClickListener.onItemClick(position, it)
            notifyDataSetChanged()
        }

        if (getPosition() == position) {
            //当前选中
            holder.rlVipItemRoot.setBackgroundResource(R.drawable.shape_vip_adapter)
            holder.tvVipItemLable.visibility = View.VISIBLE
            holder.tvVipItemTitle.setTextColor(Color.parseColor("#9C5200"))
            holder.tvVipItemCurMoney.setTextColor(Color.parseColor("#7F4D14"))
            holder.tvVipItemOldMoney.setTextColor(Color.parseColor("#B57924"))
            holder.tvVipItemMoney.setTextColor(Color.parseColor("#673810"))
            holder.tvVipItemMoney.setBackgroundResource(R.drawable.shape_vip_bottom_s)

        } else {
            holder.rlVipItemRoot.setBackgroundResource(R.drawable.shape_vip_adapter_none)
            holder.tvVipItemLable.visibility = View.GONE
            holder.tvVipItemTitle.setTextColor(Color.WHITE)
            holder.tvVipItemCurMoney.setTextColor(Color.WHITE)
            holder.tvVipItemOldMoney.setTextColor(Color.parseColor("#85898A"))
            holder.tvVipItemMoney.setTextColor(Color.parseColor("#B0B1B0"))
            holder.tvVipItemMoney.setBackgroundResource(R.drawable.shape_vip_bottom)
        }

        holder.itemView.tag = position
    }

    override fun getItemCount() = vipList.size

    interface OnItemClickListener {
        fun onItemClick(position: Int, view: View)
    }

    fun setOnItemClickListener(listener: OnItemClickListener) {
        mOnItemClickListener = listener
    }

    fun getPosition() = myPos

    fun setPosition(myPos: Int) {
        this.myPos = myPos
    }
}