package com.duben.roseplaylet.ui.fragment

import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.widget.FrameLayout
import androidx.fragment.app.Fragment
import androidx.recyclerview.widget.LinearLayoutManager
import com.bytedance.sdk.djx.DJXSdk
import com.bytedance.sdk.djx.model.DJXDramaUnlockAdMode
import com.bytedance.sdk.djx.params.DJXWidgetDramaDetailParams
import com.duben.library.utils.json.JsonUtil
import com.duben.roseplaylet.R
import com.duben.roseplaylet.ad.express.ExpressAdCallback
import com.duben.roseplaylet.ad.express.MainExpressManager
import com.duben.roseplaylet.common.AppConfig
import com.duben.roseplaylet.common.Constant
import com.duben.roseplaylet.manager.CsjDJXSdkManager
import com.duben.roseplaylet.manager.LocalVedioManager
import com.duben.roseplaylet.manager.UserManager
import com.duben.roseplaylet.mvp.model.MULTI_ITEM_1
import com.duben.roseplaylet.mvp.model.MULTI_ITEM_3
import com.duben.roseplaylet.mvp.model.VedioBean
import com.duben.roseplaylet.mvp.model.VideoMultiItemEntity3
import com.duben.roseplaylet.ui.activitys.CsjDetailActivity
import com.duben.roseplaylet.ui.adapter.NewHotStyleAdapter
import com.duben.roseplaylet.ui.fragment.base.LazyLoadBaseFragment
import com.duben.roseplaylet.utils.CommonUtils
import com.duben.roseplaylet.video.tx.newrecommend.NewTxVideoActivity
import com.scwang.smartrefresh.layout.api.RefreshLayout
import com.scwang.smartrefresh.layout.listener.OnLoadMoreListener
import kotlinx.android.synthetic.main.fragment_csjmovie.*

class CsjMovieFragment : LazyLoadBaseFragment(), OnLoadMoreListener {

    private var mType = ""

    companion object {
        private const val VIDEO_TYPE = "VIDEO_TYPE"
        private const val VIDEO_POSITION = "VIDEO_POSITION"

        fun newInstance(type: String, position: Int): Fragment {
            val args = Bundle()
            args.putString(VIDEO_TYPE, type)
            args.putInt(VIDEO_POSITION, position)
            val fragment = CsjMovieFragment()
            fragment.arguments = args
            return fragment
        }
    }

    private var hotPage = 1 // 分页
    private var hotPageSize = Constant.PAGE_SIZE // 分页

    private lateinit var hotStyleAdapter: NewHotStyleAdapter

    override fun getContentViewLayoutID() = R.layout.fragment_csjmovie

    override fun initViewsAndEvents() {

        initView()
        initListener()

        arguments?.let {
            mType = it.getString(VIDEO_TYPE, mType)
        }
    }

    override fun onResume() {
        super.onResume()
        hotPageSize = if (UserManager.getInstance().vipFlag) 12 else 9
        onRefresh()
    }

    override fun onDestroy() {
        super.onDestroy()
    }

    fun onRefresh() {
        if (AppConfig.fragmentClickFlag == Constant.FRAGMENT_CLICK_ONE) {
            hotPage = 1
            srlMainPage.resetNoMoreData()
            MainExpressManager.instance.destroy()
            loadAd()
            requestDramaByCategory()
        }
    }

    fun requestDramaByCategory() {
        CsjDJXSdkManager.requestDramaByCategory(
            mType,
            hotPage,
            hotPageSize,
            object : CsjDJXSdkManager.CsjrequestDramaByCategoryListener {
                override fun dramaCategoryListSuccess(list: MutableList<VedioBean>) {
                    if (activity == null || requireActivity().isFinishing) {
                        srlMainPage?.finishRefresh(true)
                        return
                    }

                    if (::hotStyleAdapter.isInitialized) {
                        if (hotPage == 1) {
                            srlMainPage.finishRefresh(true)
                            hotStyleAdapter.setNewInstance(getAdapterData(list))
                        } else {
                            if (list.size < hotPageSize) {
                                srlMainPage.finishLoadMoreWithNoMoreData()
                            } else {
                                loadAd()
                                srlMainPage.finishLoadMore()
                            }
                            hotStyleAdapter.addData(getAdapterData(list))
                        }
                    }
                }

                override fun dramaCategoryListFail() {

                }

            })
    }

    override fun onLoadMore(refreshLayout: RefreshLayout) {
        hotPage = ++hotPage
        requestDramaByCategory()
    }

    private fun initView() {
        rv_pic_list.layoutManager = LinearLayoutManager(context)
        hotStyleAdapter = NewHotStyleAdapter()
        hotStyleAdapter.animationEnable = false
        rv_pic_list.adapter = hotStyleAdapter
        val emptyView =
            LayoutInflater.from(requireContext()).inflate(R.layout.item_empty_data, null)
        hotStyleAdapter.setEmptyView(emptyView)
        hotStyleAdapter.setOnCustomChildClickListener(object :
            NewHotStyleAdapter.OnCustomChildClickListener {
            override fun onCustomChildClick(view: View, vedioBean: VedioBean) {
                requestDrama(vedioBean)
            }
        })
    }

    fun requestDrama(data: VedioBean) {
        CsjDJXSdkManager.requestDrama(data.thirdId.toLong()) {
            val bundle = Bundle()
            val vedioBean = VedioBean()
            // 免费集数
            vedioBean.freeIndex = it.freeSet
            vedioBean.groupId = it.groupId
            bundle.putString(Constant.VEDIO_BEAN, JsonUtil.toJson(vedioBean))
            CsjDetailActivity.outerDrama = it
            CsjDetailActivity.enterFrom =
                DJXWidgetDramaDetailParams.DJXDramaEnterFrom.DEFAULT
            readyGo(CsjDetailActivity::class.java, bundle)
        }
    }

    private fun initListener() {
        srlMainPage.setEnableRefresh(false)
        srlMainPage.setOnLoadMoreListener(this)
    }

    private fun getAdapterData(list: List<VedioBean>): MutableList<VideoMultiItemEntity3> {
        val data = mutableListOf<VideoMultiItemEntity3>()
        val subList = CommonUtils.getSubList(3, list)
        for (itemEntity3s in subList) {
            data.add(VideoMultiItemEntity3(MULTI_ITEM_1, itemEntity3s, null))
        }
        return data
    }

    private fun loadAd() {
        MainExpressManager.instance.loadAd(object : ExpressAdCallback {
            override fun loadSuccess(adView: FrameLayout?) {

            }

            override fun renderSuccess(adView: FrameLayout?): Boolean {
                adView?.let { addAdView(it) }
                return false
            }

            override fun loadFail() {
            }
        })
    }

    private fun addAdView(adView: FrameLayout) {
        // 默认每各9个加1个
        var insertIndex = 3

        val allData = hotStyleAdapter.data
        if (allData != null && allData.size > 0) {
            for (i in allData.size - 1 downTo 0) {
                if (allData[i].itemType == MULTI_ITEM_3) {
                    insertIndex += 4
                }
            }
            if (insertIndex >= allData.size) {
                insertIndex = allData.size
            }

            if (allData[allData.lastIndex].itemType == MULTI_ITEM_3) return

            val data = VideoMultiItemEntity3(MULTI_ITEM_3, null, adView)
            hotStyleAdapter.addData(insertIndex, data)
        }
    }

    override fun onDestroyView() {
        super.onDestroyView()
        MainExpressManager.instance.destroy()
    }

}