package com.duben.roseplaylet.ui.fragment

import android.os.Bundle
import android.os.Handler
import android.os.Looper
import android.text.TextUtils
import android.view.LayoutInflater
import android.view.View
import android.widget.TextView
import androidx.core.content.ContextCompat
import androidx.fragment.app.Fragment
import androidx.recyclerview.widget.RecyclerView
import androidx.viewpager2.widget.ViewPager2
import com.duben.library.utils.json.JsonUtil
import com.duben.library.utils.nodoubleclick.AntiShake
import com.duben.roseplaylet.R
import com.duben.roseplaylet.common.AppConfig
import com.duben.roseplaylet.common.Constant
import com.duben.roseplaylet.manager.CsjDJXSdkManager
import com.duben.roseplaylet.manager.LocalVedioManager
import com.duben.roseplaylet.manager.UserManager
import com.duben.roseplaylet.mvp.model.HotStyleTypesBean
import com.duben.roseplaylet.ui.adapter.CsjVideoPageAdapter
import com.duben.roseplaylet.ui.fragment.base.LazyLoadBaseFragment
import com.duben.roseplaylet.utils.LogUtil
import com.google.android.material.tabs.TabLayout
import com.google.android.material.tabs.TabLayoutMediator
import com.scwang.smartrefresh.layout.api.RefreshLayout
import com.scwang.smartrefresh.layout.listener.OnRefreshListener
import kotlinx.android.synthetic.main.fragment_csjvedio.*
import java.lang.reflect.Field
import java.util.*
import kotlin.concurrent.schedule

/**
 * 描述：首页
 * 作者：孟崔广
 * 时间：2023/6/29 11:30
 */
class CsjVedioFragment : LazyLoadBaseFragment(), View.OnClickListener, OnRefreshListener {

    companion object {
        fun newInstance(): Fragment {
            val args = Bundle()
            val fragment = CsjVedioFragment()
            fragment.arguments = args
            return fragment
        }
    }

    var mSelectTabIndex = -1
    private val tabsData = mutableListOf<HotStyleTypesBean>()
    private var vpAdapter: CsjVideoPageAdapter? = null
    private var alipayTime = 0


    private val userManager by lazy { UserManager.getInstance() }

    override fun getContentViewLayoutID() = R.layout.fragment_csjvedio

    override fun initViewsAndEvents() {
    }

    override fun onFragmentFirstVisible() {
        initVp()
        initListener()
        loadData()
    }

    override fun onDestroy() {
        super.onDestroy()
    }

    override fun onFragmentResume() {
        if (AppConfig.fragmentClickFlag == Constant.FRAGMENT_CLICK_ONE) {

        }
    }

    override fun onFragmentPause() {
    }

    private fun initListener() {
        srlMainPage.setOnRefreshListener(this)
        iv_main_watching_close.setOnClickListener(this)
        tv_main_watching_goto.setOnClickListener(this)
    }

    override fun onClick(v: View?) {
        if (AntiShake.check(v?.id)) return
        when (v?.id) {
            R.id.iv_main_watching_close -> {
                LocalVedioManager.closeCacheVedio()
                ll_main_watching_root.visibility = View.GONE
            }
            R.id.tv_main_watching_goto -> {
                val cacheVedio = LocalVedioManager.getCacheVedio()
                if (cacheVedio != null) {
                    LocalVedioManager.startVedioDetailActivityForType(requireActivity(), cacheVedio)
                }
            }
        }
    }

    private fun loadData() {
        Timer().schedule(200) {
            requireActivity().runOnUiThread {

                if (!TextUtils.isEmpty(userManager?.userID)) {
                    CsjDJXSdkManager.requestDramaCategoryList(object : CsjDJXSdkManager.CsjdramaCategoryListListener {
                        override fun dramaCategoryListSuccess(list: MutableList<HotStyleTypesBean>) {
                            tabsData.clear()
                            tabsData.addAll(list)
                            LogUtil.d("mcg--->"+JsonUtil.toJson(list))
                            if (tabsData.isNotEmpty()) {
                                mSelectTabIndex = 0

                                vpAdapter?.run {
                                    notifyDataSetChanged()

                                    Handler(Looper.getMainLooper()).postDelayed({
                                        if (getFragments().isNotEmpty()) {
                                            val fragment = getFragments()[mSelectTabIndex]
                                            if (fragment != null) {
                                                (fragment as CsjMovieFragment).onRefresh()
                                            }
                                        }
                                    }, 200)
                                }
                            }
                        }

                        override fun dramaCategoryListFail() {

                        }

                    })
                }
            }
        }
    }

    private fun initVp() {
        vpAdapter = CsjVideoPageAdapter(tabsData, this)
        vp2.adapter = vpAdapter
        vp2.desensitization()
        TabLayoutMediator(tablayout, vp2) { tab, position ->
            // 初始化Tab
            tab.id = position
            if (tabsData.size > 0) {
                tab.customView = getTabView(tabsData[position].type)
            }
        }.attach()
        tablayout.addOnTabSelectedListener(object : TabLayout.OnTabSelectedListener {
            override fun onTabSelected(tab: TabLayout.Tab?) {
                updateTab(tab, true)
                mSelectTabIndex = tab!!.id
            }

            override fun onTabUnselected(tab: TabLayout.Tab?) {
                updateTab(tab, false)
            }

            override fun onTabReselected(tab: TabLayout.Tab?) {
            }
        })
        vp2.offscreenPageLimit = 100
    }

    private fun updateTab(tab: TabLayout.Tab?, isSelected: Boolean) {
        tab?.customView?.let {
            val text = it.findViewById<TextView>(R.id.item_tv)
            if (isSelected) {
                text.setTextColor(ContextCompat.getColor(requireContext(), R.color.red))
            } else {
                text.setTextColor(ContextCompat.getColor(requireContext(), R.color.color_8D8F90))
            }
        }
    }

    private fun getTabView(text: String): View {
        val view = LayoutInflater.from(requireContext()).inflate(R.layout.item_video_tab, null)
        view.findViewById<TextView>(R.id.item_tv).text = text
        return view
    }

    override fun onRefresh(refreshLayout: RefreshLayout) {
        vpAdapter?.run {
            if (getFragments().isNotEmpty()) {
                val fragment = getFragments()[mSelectTabIndex]
                if (fragment != null) {
                    (fragment as CsjMovieFragment).onRefresh()
                    refreshLayout.finishRefresh()
                }
            }
        }
    }

    override fun onFragmentResume(firstResume: Boolean) {
        super.onFragmentResume(firstResume)

        vpAdapter?.getFragments()?.get(mSelectTabIndex)?.onResume()
    }

    override fun onPause() {
        super.onPause()
        vpAdapter?.getFragments()?.get(mSelectTabIndex)?.onPause()
    }

    fun ViewPager2.desensitization() {
        //动态设置ViewPager2 灵敏度
        try {
            val recyclerViewField: Field = ViewPager2::class.java.getDeclaredField("mRecyclerView")
            recyclerViewField.isAccessible = true
            val recyclerView = recyclerViewField.get(this) as RecyclerView
            val touchSlopField: Field = RecyclerView::class.java.getDeclaredField("mTouchSlop")
            touchSlopField.isAccessible = true
            val touchSlop = touchSlopField.get(recyclerView) as Int
            touchSlopField.set(recyclerView, touchSlop * 3) //6 is empirical value
        } catch (ignore: java.lang.Exception) {
        }
    }
}