package com.duben.roseplaylet.ui.fragment

import android.text.TextUtils
import android.view.LayoutInflater
import android.view.View
import android.widget.TextView
import androidx.core.content.ContextCompat
import androidx.fragment.app.Fragment
import com.duben.roseplaylet.R
import com.duben.roseplaylet.common.AppConfig
import com.duben.roseplaylet.common.Constant
import com.duben.roseplaylet.manager.UserManager
import com.duben.roseplaylet.mvp.presenters.FirstPresenter
import com.duben.roseplaylet.mvp.views.FirstView
import com.duben.roseplaylet.ui.adapter.RecommendPageAdapter
import com.duben.roseplaylet.ui.fragment.base.LazyLoadBaseFragment
import com.google.android.material.tabs.TabLayout
import com.google.android.material.tabs.TabLayoutMediator
import kotlinx.android.synthetic.main.fragment_first.*

/**
 * 短剧首页
 *
 * 孟崔广
 */
class FirstFragment : LazyLoadBaseFragment(), FirstView {

    companion object {
        var currentPosition = 0
    }

    private var mSelectTabIndex = 0
    private val tabsData = mutableListOf<String>()
    private val fragments = mutableListOf<Fragment>()
    private var vpAdapter: RecommendPageAdapter? = null

    private val firstPresenter by lazy { FirstPresenter() }

    private val userManager by lazy { UserManager.getInstance() }

    override fun getContentViewLayoutID() = R.layout.fragment_first

    override fun initViewsAndEvents() {
        firstPresenter.attachView(this)

        initVp2()
    }

    override fun onFragmentFirstVisible() {

    }

    override fun onFragmentResume() {
        if (AppConfig.fragmentClickFlag == Constant.FRAGMENT_CLICK_ONE) {

            if (TextUtils.isEmpty(userManager?.userID)) {
                firstPresenter.userLogin()
            }
        }
    }

    override fun onDestroy() {
        super.onDestroy()
        firstPresenter.detachView()
    }

    private fun initVp2() {
        tabsData.add("推 荐")
        tabsData.add("VIP专享")
        fragments.clear()
        fragments.add(CsjVedioFragment.newInstance())
        fragments.add(MainFragment.newInstance())
        vpAdapter = RecommendPageAdapter(fragments, this)
        vp2_recommend.adapter = vpAdapter
        TabLayoutMediator(tab_recommend, vp2_recommend) { tab, position ->
            tab.id = position
            tab.customView = getTabView(position)
        }.attach()
        tab_recommend.addOnTabSelectedListener(object : TabLayout.OnTabSelectedListener {
            override fun onTabSelected(tab: TabLayout.Tab?) {
                currentPosition = tab!!.position
                mSelectTabIndex = tab!!.id
                updateTab(tab, true, mSelectTabIndex)
            }

            override fun onTabUnselected(tab: TabLayout.Tab?) {
                updateTab(tab, false, mSelectTabIndex)
            }

            override fun onTabReselected(tab: TabLayout.Tab?) {
            }
        })
        tab_recommend.getTabAt(0)?.select()
        vp2_recommend.setCurrentItem(0, false)
        vp2_recommend.offscreenPageLimit = 1
        vp2_recommend.isUserInputEnabled = false
    }

    private fun getTabView(position: Int): View {
        val view = LayoutInflater.from(requireContext()).inflate(R.layout.item_recommend_tab, null)
        view.findViewById<TextView>(R.id.item_tv).text = tabsData[position]
        if (position == 0) {
            view.findViewById<View>(R.id.item_line).background =
                ContextCompat.getDrawable(requireContext(), R.drawable.shape_line_corner_red)
        }
        return view
    }

    private fun updateTab(tab: TabLayout.Tab?, isSelected: Boolean, position: Int) {
        tab?.customView?.let {
            val text = it.findViewById<TextView>(R.id.item_tv)
            val line = it.findViewById<View>(R.id.item_line)
            if (isSelected) {
                line.visibility = View.VISIBLE
                text.setTextColor(ContextCompat.getColor(requireContext(), R.color.black))
            } else {
                text.setTextColor(
                    ContextCompat.getColor(
                        requireContext(),
                        R.color.black
                    )
                )
                line.visibility = View.INVISIBLE
            }
            line.background =
                ContextCompat.getDrawable(requireContext(), R.drawable.shape_line_corner_red)
        }
    }
}