package com.duben.roseplaylet.ui.fragment

import android.os.Bundle
import android.os.Handler
import android.os.Looper
import android.text.TextUtils
import android.view.LayoutInflater
import android.view.View
import android.widget.TextView
import androidx.core.content.ContextCompat
import androidx.fragment.app.Fragment
import androidx.recyclerview.widget.LinearLayoutManager
import androidx.recyclerview.widget.RecyclerView
import androidx.viewpager2.widget.ViewPager2
import com.duben.library.utils.GlideUtils
import com.duben.library.utils.nodoubleclick.AntiShake
import com.duben.roseplaylet.R
import com.duben.roseplaylet.ad.AdManager
import com.duben.roseplaylet.ad.banner.BannerManager
import com.duben.roseplaylet.common.AppConfig
import com.duben.roseplaylet.common.Constant
import com.duben.roseplaylet.manager.LocalVedioManager
import com.duben.roseplaylet.manager.TrackManager
import com.duben.roseplaylet.manager.UserManager
import com.duben.roseplaylet.mvp.model.*
import com.duben.roseplaylet.mvp.presenters.HomePresenter
import com.duben.roseplaylet.mvp.views.HomeView
import com.duben.roseplaylet.ui.activitys.AlipayTimeActivity
import com.duben.roseplaylet.ui.activitys.NineActivity
import com.duben.roseplaylet.ui.activitys.VipActivity
import com.duben.roseplaylet.ui.adapter.HomeVideoPageAdapter
import com.duben.roseplaylet.ui.adapter.ImageTitleAdapter
import com.duben.roseplaylet.ui.adapter.TopAdapter
import com.duben.roseplaylet.ui.fragment.base.LazyLoadBaseFragment
import com.duben.roseplaylet.ui.widgets.SimpleCountDownTimer
import com.duben.roseplaylet.utils.AppPreferencesManager
import com.google.android.material.tabs.TabLayout
import com.google.android.material.tabs.TabLayoutMediator
import com.scwang.smartrefresh.layout.api.RefreshLayout
import com.scwang.smartrefresh.layout.listener.OnRefreshListener
import kotlinx.android.synthetic.main.fragment_main.*
import java.lang.reflect.Field
import java.util.*
import kotlin.concurrent.schedule

/**
 * 描述：首页
 * 作者：孟崔广
 * 时间：2023/6/29 11:30
 */
class MainFragment : LazyLoadBaseFragment(), HomeView, View.OnClickListener, OnRefreshListener {

    companion object {

        fun newInstance(): Fragment {
            val args = Bundle()
            val fragment = MainFragment()
            fragment.arguments = args
            return fragment
        }
    }
    var mSelectTabIndex = -1
    private val tabsData = mutableListOf<HotStyleTypesBean>()
    private val bannerList = mutableListOf<BannerBean>()
    private val topList = mutableListOf<VedioBean>()
    private var vpAdapter: HomeVideoPageAdapter? = null
    private var topAdapter: TopAdapter? = null
    private var recommendVedioBean: RecoBean? = null
    private var alipayTime = 0

    private val homePresenter by lazy { HomePresenter() }
    private val userManager by lazy { UserManager.getInstance() }

    override fun getContentViewLayoutID() = R.layout.fragment_main

    override fun initViewsAndEvents() {
        homePresenter.attachView(this)
    }

    override fun onFragmentFirstVisible() {
        initVp()
        initListener()
        loadData()
    }

    /**
     * 跳转到主页面
     */
    private fun goToMainActivity() {
        homePresenter.getSoltVedio()
    }

    override fun onDestroy() {
        super.onDestroy()
        homePresenter.detachView()
    }

    override fun onFragmentResume() {
        if (AppConfig.fragmentClickFlag == Constant.FRAGMENT_CLICK_ONE) {
            if (AntiShake.check(banner?.id)) return

            BannerManager.loadAd(requireActivity(), fl_main_banner)

            banner?.start()
            setWatchingStatus()

            if (TextUtils.isEmpty(userManager?.userID)) {
                //获取游客登录数据
                homePresenter.userLogin()
            } else {
                homePresenter.topTabs()
                homePresenter.orders()
                if (userManager.vipFlag) {
                    homePresenter.signAfterSeconds()
                }
                TrackManager.getInstance().getMyInfo()
            }

            AppConfig.showVipAdDialog = false
        }
    }

    override fun onFragmentPause() {
        banner?.stop()
    }

    private fun initListener() {
        srlMainPage.setOnRefreshListener(this)
        iv_main_watching_close.setOnClickListener(this)
        tv_main_watching_goto.setOnClickListener(this)
        ll_lucky.setOnClickListener(this)
        iv_kefu_main.setOnClickListener(this)

//        context?.let {
//            GlideUtils.loadImageViewGifForCenterCrop(
//                it,
//                "https://mints-pkg.oss-cn-beijing.aliyuncs.com/pkg/img/ic_kefu_main.png",
//                iv_kefu_main,
//                1
//            )
//        }
    }

    override fun onClick(v: View?) {
        if (AntiShake.check(v?.id)) return
        when (v?.id) {
            R.id.iv_kefu_main -> {
                val bundle = Bundle()
                bundle.putInt(AlipayTimeActivity.ALIPAY_TIME, alipayTime)
                readyGo(AlipayTimeActivity::class.java, bundle)
            }
            R.id.iv_main_watching_close -> {
                LocalVedioManager.closeCacheVedio()
                ll_main_watching_root.visibility = View.GONE
            }
            R.id.tv_main_watching_goto -> {
                val cacheVedio = LocalVedioManager.getCacheVedio()
                if (cacheVedio != null) {
                    LocalVedioManager.startVedioDetailActivityForType(requireActivity(), cacheVedio)
                }
            }
            R.id.ll_lucky -> {
                readyGo(NineActivity::class.java)
            }
        }
    }


    private fun loadData() {
        Timer().schedule(200) {
            requireActivity().runOnUiThread {
                if (AntiShake.check(rv_main_list?.id)) return@runOnUiThread

                if (!TextUtils.isEmpty(userManager?.userID)) {
                    homePresenter.getHomeV1Types()
                }
            }
        }
    }

    private fun initVp() {
        topAdapter = TopAdapter(topList)
        val ma = LinearLayoutManager(requireContext())
        ma.orientation = LinearLayoutManager.HORIZONTAL
        rv_main_list.layoutManager = ma
        rv_main_list.adapter = topAdapter
        topAdapter?.setOnItemClickListener(object : TopAdapter.OnItemClickListener {
            override fun onItemClick(position: Int) {
                if (topList.size > 0) {
                    LocalVedioManager.startVedioDetailActivityForType(
                        requireActivity(),
                        topList[position]
                    )
                }
            }
        })

        vpAdapter = HomeVideoPageAdapter(tabsData, this)
        vp2.adapter = vpAdapter
        vp2.desensitization()
        TabLayoutMediator(tablayout, vp2) { tab, position ->
            // 初始化Tab
            tab.id = position
            if (tabsData.size > 0) {
                tab.customView = getTabView(tabsData[position].type)
            }
        }.attach()
        tablayout.addOnTabSelectedListener(object : TabLayout.OnTabSelectedListener {
            override fun onTabSelected(tab: TabLayout.Tab?) {
                updateTab(tab, true)
                mSelectTabIndex = tab!!.id
            }

            override fun onTabUnselected(tab: TabLayout.Tab?) {
                updateTab(tab, false)
            }

            override fun onTabReselected(tab: TabLayout.Tab?) {
            }
        })
        vp2.offscreenPageLimit = 100
    }

    private fun updateTab(tab: TabLayout.Tab?, isSelected: Boolean) {
        tab?.customView?.let {
            val text = it.findViewById<TextView>(R.id.item_tv)
            if (isSelected) {
                text.setTextColor(ContextCompat.getColor(requireContext(), R.color.red))
            } else {
                text.setTextColor(ContextCompat.getColor(requireContext(), R.color.color_8D8F90))
            }
        }
    }

    private fun getTabView(text: String): View {
        val view = LayoutInflater.from(requireContext()).inflate(R.layout.item_video_tab, null)
        view.findViewById<TextView>(R.id.item_tv).text = text
        return view
    }

    override fun onRefresh(refreshLayout: RefreshLayout) {
        vpAdapter?.run {
            if (getFragments().isNotEmpty()) {
                val fragment = getFragments()[mSelectTabIndex]
                if (fragment != null) {
                    (fragment as MovieFragment).onRefresh()
                    refreshLayout.finishRefresh()
                }
            }
        }
    }


    fun setWatchingStatus() {
        if (LocalVedioManager.isCacheVedio()) {
            ll_main_watching_root.visibility = View.VISIBLE
            ll_main_watching_root.setOnClickListener { }
            val cacheVedio = LocalVedioManager.getCacheVedio()
            if (cacheVedio != null) {
                GlideUtils.loadImageViewGifForCenterCrop(
                    requireContext(),
                    cacheVedio.coverImage,
                    iv_main_watching_pic
                )
                tv_main_watching_name.text = cacheVedio.title
                tv_main_watching_text1.text = "上次观看至第${cacheVedio.seeIndex}集"
                if (cacheVedio.orderTags != null && cacheVedio.orderTags.size > 0) {
                    val sb = StringBuilder()
                    for (i in 0 until cacheVedio.orderTags.size) {
                        sb.append(cacheVedio.orderTags.get(i) + " ")
                    }
                    tv_main_watching_text2.text = "${sb}"
                }
            }
        } else {
            ll_main_watching_root.visibility = View.GONE
        }
    }

    override fun getHomeV1MsgSuc(data: HotStyleTypesList) {
        tabsData.clear()
        tabsData.addAll(data.list)
        if (tabsData.isNotEmpty()) {
            mSelectTabIndex = 0

            vpAdapter?.run {
                notifyDataSetChanged()

                Handler(Looper.getMainLooper()).postDelayed({
                    if (getFragments().isNotEmpty()) {
                        val fragment = getFragments()[mSelectTabIndex]
                        if (fragment != null) {
                            (fragment as MovieFragment).onRefresh()
                        }
                    }
                }, 200)
            }
        }
    }

    override fun topTabsSuc(data: BannerList?) {
        if (data != null && data.list != null && data.list.size > 0) {
            bannerList.clear()
            for (i in 0 until data.list.size) {
                val bean = BannerBean()
                bean.title = data.list[i].topTabsReamrk
                bean.imageRes = data.list[i].typeImg
                bannerList.add(bean)
            }

            banner.addBannerLifecycleObserver(this)
                .setAdapter(ImageTitleAdapter(bannerList))
                .setOnBannerListener { data2, position ->
                    val banner = data.list[position]
                    if (banner.createType == 4) {
                        // 九宫格
                        val bundle = Bundle()
                        bundle.putBoolean(NineActivity.IS_AD_SHOW, true)
                        readyGo(NineActivity::class.java, bundle)
                    } else {
                        LocalVedioManager.startVedioDetailActivityForType(requireActivity(), banner)
                    }

                }
        }
    }

    override fun ordersSuc(data: BannerList?) {
        topList.clear()
        if (data != null) {
            topList.addAll(data.list)
        }
        topAdapter?.notifyDataSetChanged()
    }

    override fun getSoltVedioSuc(data: IndexList?) {
//        val firstVedio = AppPreferencesManager.get().getBoolean(Constant.IS_FIRST_VEDIO, true)
//        if (data != null && data.vedioMsg != null && firstVedio) {
//            AppPreferencesManager.get().put(Constant.IS_FIRST_VEDIO, false)
//            LocalVedioManager.startVedioDetailActivityForType(requireActivity(), data.vedioMsg)
//        } else {
//            if (UserManager.getInstance().newFlag && !UserManager.getInstance().vipFlag) {
////                homePresenter.getRecommendVedio()
////
//                val bundle = Bundle()
//                bundle.putBoolean(VipActivity.IS_MAIN, true)
//                readyGo(VipActivity::class.java, bundle)
//            }
//        }
    }

    override fun getRecommendVedioSuc(data: RecoBean?) {
        this.recommendVedioBean = data
    }

    override fun signAfterSecondsSuc(time: Int) {
        this.alipayTime = time
//        if (countDownTimer != null) {
//            countDownTimer!!.cancel()
//            countDownTimer = null
//        }
        if (time > 0) {
            iv_kefu_main.visibility = View.VISIBLE
//            countDownTimer =
//                SimpleCountDownTimer(time.toLong() * 1000, tv_alipaytime).setOnFinishListener {
//                    iv_kefu_main?.visibility = View.GONE
//                }.start() as SimpleCountDownTimer?

        } else {
            iv_kefu_main.visibility = View.GONE
        }
    }

    override fun onFragmentResume(firstResume: Boolean) {
        super.onFragmentResume(firstResume)

        vpAdapter?.getFragments()?.get(mSelectTabIndex)?.onResume()
    }

    override fun onPause() {
        super.onPause()
        vpAdapter?.getFragments()?.get(mSelectTabIndex)?.onPause()
    }

    fun ViewPager2.desensitization() {
        //动态设置ViewPager2 灵敏度
        try {
            val recyclerViewField: Field = ViewPager2::class.java.getDeclaredField("mRecyclerView")
            recyclerViewField.isAccessible = true
            val recyclerView = recyclerViewField.get(this) as RecyclerView
            val touchSlopField: Field = RecyclerView::class.java.getDeclaredField("mTouchSlop")
            touchSlopField.isAccessible = true
            val touchSlop = touchSlopField.get(recyclerView) as Int
            touchSlopField.set(recyclerView, touchSlop * 3) //6 is empirical value
        } catch (ignore: java.lang.Exception) {
        }
    }
}