package com.duben.roseplaylet.ui.fragment

import android.view.LayoutInflater
import android.view.View
import android.widget.TextView
import androidx.core.content.ContextCompat
import androidx.fragment.app.Fragment
import com.duben.library.utils.nodoubleclick.AntiShake
import com.duben.roseplaylet.R
import com.duben.roseplaylet.common.AppConfig
import com.duben.roseplaylet.common.Constant
import com.duben.roseplaylet.manager.UserManager
import com.duben.roseplaylet.ui.adapter.MsgPageAdapter
import com.duben.roseplaylet.ui.fragment.base.LazyLoadBaseFragment
import com.google.android.material.tabs.TabLayout
import com.google.android.material.tabs.TabLayoutMediator
import com.tencent.qcloud.tuikit.tuiconversation.classicui.page.TUIConversationFragment
import kotlinx.android.synthetic.main.fragment_main_message.*

class MessageFragment : LazyLoadBaseFragment(), View.OnClickListener {

    private val tabsData = arrayListOf<String>()
    private val fragmentsData = mutableListOf<Fragment>()
    private var vpAdapter: MsgPageAdapter? = null
    private var mSelectTabIndex = 0

    private val userManager by lazy { UserManager.getInstance() }

    override fun getContentViewLayoutID() = R.layout.fragment_main_message

    override fun initViewsAndEvents() {
        initView()
    }

    private fun initView() {
        tabsData.add(String.format("聊天"))
        tabsData.add(String.format("系统消息"))

        fragmentsData.add(TUIConversationFragment())
        fragmentsData.add(SystemMsgFragment())

        vpAdapter = MsgPageAdapter(tabsData, fragmentsData, requireActivity())
        vp2.adapter = vpAdapter

        TabLayoutMediator(tab_msg, vp2) { tab, position ->
            // 初始化Tab
            tab.id = position
            if (tabsData.size > 0) {
                tab.customView = getTabView(tabsData[position])
            }
        }.attach()
        tab_msg.addOnTabSelectedListener(object : TabLayout.OnTabSelectedListener {
            override fun onTabSelected(tab: TabLayout.Tab?) {
                updateTab(tab, true)
                mSelectTabIndex = tab!!.id
            }

            override fun onTabUnselected(tab: TabLayout.Tab?) {
                updateTab(tab, false)
            }

            override fun onTabReselected(tab: TabLayout.Tab?) {
            }
        })
    }

    private fun getTabView(text: String): View {
        val view = LayoutInflater.from(requireContext()).inflate(R.layout.item_video_tab, null)
        view.findViewById<TextView>(R.id.item_tv).text = text
        return view
    }

    private fun updateTab(tab: TabLayout.Tab?, isSelected: Boolean) {
        tab?.customView?.let {
            val text = it.findViewById<TextView>(R.id.item_tv)
            if (isSelected) {
                text.setTextColor(ContextCompat.getColor(requireContext(), R.color.red))
            } else {
                text.setTextColor(ContextCompat.getColor(requireContext(), R.color.color_8D8F90))
            }
        }
    }


    override fun onFragmentResume() {
        super.onFragmentResume()
        if (AppConfig.fragmentClickFlag == Constant.FRAGMENT_CLICK_FOUR) {

        }
    }

    override fun onClick(v: View?) {
        if (AntiShake.check(v?.id)) return

        when (v?.id) {

        }
    }
}