package com.duben.roseplaylet.ui.fragment

import android.content.Intent
import android.text.TextUtils
import android.view.View
import android.widget.FrameLayout
import com.duben.library.utils.nodoubleclick.AntiShake
import com.duben.roseplaylet.MintsApplication
import com.duben.roseplaylet.R
import com.duben.roseplaylet.ad.express.ExpressAdCallback
import com.duben.roseplaylet.ad.express.MyExpressManager
import com.duben.roseplaylet.common.AppConfig
import com.duben.roseplaylet.common.Constant
import com.duben.roseplaylet.manager.GenerateTestUserSig
import com.duben.roseplaylet.manager.LoginWrapper
import com.duben.roseplaylet.manager.UserManager
import com.duben.roseplaylet.mvp.model.UserBean
import com.duben.roseplaylet.mvp.presenters.MyPresenter
import com.duben.roseplaylet.mvp.views.MyView
import com.duben.roseplaylet.ui.activitys.*
import com.duben.roseplaylet.ui.fragment.base.LazyLoadBaseFragment
import com.duben.roseplaylet.utils.LogUtil
import com.duben.roseplaylet.utils.SpanUtils
import com.duben.roseplaylet.utils.TimeRender
import com.duben.roseplaylet.utils.UIUtils
import com.tencent.imsdk.v2.V2TIMConversation
import com.tencent.qcloud.tuicore.TUIConstants
import com.tencent.qcloud.tuicore.interfaces.TUICallback
import kotlinx.android.synthetic.main.fragment_main_my.*

/**
 * 描述：我
 * 作者：孟崔广
 * 时间：2023/7/7 11:30
 */
class MyFragment : LazyLoadBaseFragment(), MyView, View.OnClickListener {

    private val myPresenter by lazy { MyPresenter() }
    private val userManager by lazy { UserManager.getInstance() }

    override fun getContentViewLayoutID() = R.layout.fragment_main_my

    override fun initViewsAndEvents() {
        myPresenter.attachView(this)

        tv_contact.text = SpanUtils()
            .append("如有问题，请直接联系")
            .append("在线客服\n")
            .setForegroundColor(resources.getColor(R.color.main_mints))
            .append("处理速度更快，")
            .append("无需联系支付宝")
            .setForegroundColor(resources.getColor(R.color.main_mints))
            .create()

        initListener()
    }

    override fun onFragmentFirstVisible() {
        super.onFragmentFirstVisible()
    }

    override fun onDestroy() {
        super.onDestroy()
        myPresenter.detachView()
    }

    override fun onFragmentResume() {
        super.onFragmentResume()

        if (AppConfig.fragmentClickFlag == Constant.FRAGMENT_CLICK_FIVE) {
            if (AntiShake.check(ll_my_record?.id)) return

            if (userManager.vipFlag) {
                MyExpressManager.instance.preLoadAd()
                showAd()
            }

            if (!TextUtils.isEmpty(userManager?.userID)) {
                myPresenter.getMyInfo()
            } else {
                myPresenter.userLogin()
            }
        }
    }

    private fun startChatActivity() {
        val intent = Intent(MintsApplication.getContext(), IMActivity::class.java)
        intent.putExtra(TUIConstants.TUIChat.CHAT_TYPE, V2TIMConversation.V2TIM_C2C)
        intent.putExtra(TUIConstants.TUIChat.CHAT_ID, userManager.userID)
        intent.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK)
        startActivity(intent)
    }

    private fun initUserLocalData() {
        val instance = UserManager.getInstance()
        val tuiLoginConfig = LoginWrapper.getLoginConfig()
        tuiLoginConfig.isInitLocalStorageOnly = true
        val userID = instance.userID
        val userSig = GenerateTestUserSig.genTestUserSig(userID)

        LoginWrapper.getInstance().loginIMSDK(MintsApplication.getContext(),
            AppConfig.IM_SDK_APPID, userID, userSig, tuiLoginConfig, object : TUICallback() {
                override fun onSuccess() {
                    LogUtil.i("LoginWrapper ", "imLogin onSuccess")
                    startChatActivity()
                }

                override fun onError(errorCode: Int, errorMessage: String) {
                    LogUtil.i(
                        "LoginWrapper ",
                        "imLogin errorCode = $errorCode, errorInfo = $errorMessage"
                    )
                }
            })
    }

    override fun onClick(v: View?) {
        if (AntiShake.check(v?.id)) return

        when (v?.id) {
            R.id.tv_my_btn -> {
//                if (!userManager.vipFlag) {
//                    readyGo(VipActivity::class.java)
//                }
            }
            R.id.ll_my_setting -> {
//                readyGo(SettingsActivity::class.java)
                readyGo(RealAuthActivity::class.java)
            }
            R.id.tv_my_account, R.id.ll_my_login -> {
                if (!userManager.userIsLogin()) {
                    readyGo(MobileLoginActivity::class.java)
                }
            }
            R.id.ll_my_record -> {
//                readyGo(WatchRecordActivity::class.java)
                readyGo(UserProfileActivity::class.java)
            }
            R.id.ll_my_feed -> {
//                readyGo(FeedbackActivity::class.java)
                readyGo(UserLabelActivity::class.java)
            }
            R.id.ll_contactus -> {
//                (requireActivity() as MainActivity).backPhoneDialog()

                initUserLocalData()
            }
            R.id.ll_my_payrecord -> {
                readyGo(OrderRecordActivity::class.java)
            }
        }
    }

    override fun getUserSuc(data: UserBean) {
        loginStatus(data)
    }

    private fun loginStatus(consumer: UserBean) {
        // 1-匹配用户
//        if (consumer.activiteFlag == 1) {
//            // 匹配
//            ll_my_vip.visibility = View.VISIBLE
//            ll_my_payrecord.visibility = View.VISIBLE
//            ll_contactus.visibility = View.VISIBLE
//            line_my_payrecord.visibility = View.VISIBLE
//            line_my_contactus.visibility = View.VISIBLE
//        } else {
//            // 未匹配
//            ll_my_vip.visibility = View.GONE
//            ll_my_payrecord.visibility = View.GONE
//            ll_contactus.visibility = View.GONE
//            line_my_payrecord.visibility = View.GONE
//            line_my_contactus.visibility = View.GONE
//        }

        // 2-是否有手机号
        if (!TextUtils.isEmpty(userManager.mobile)) {
            tv_my_account.text =
                "手机号:" + consumer.mobile.substring(0, 3) + "****" + consumer.mobile.substring(
                    consumer.mobile.length - 4
                )
            iv_my_bind.visibility = View.GONE
        } else {
            tv_my_account.text = "绑定手机号"
            iv_my_bind.visibility = View.VISIBLE
        }
        tv_my_id.text = "用户ID:" + consumer.idcode

        // 3-vip
        if (consumer.expireTime > 0) {
            tv_my_vip.text = "VIP会员"
            tv_my_btn.visibility = View.GONE
            if (consumer.isForever) {
                tv_my_status.text = "有效期: 永久有效"
            } else {
                tv_my_status.text =
                    "到期时间 ${TimeRender.formatDate(consumer.expireTime, TimeRender.DEFAULT_FORMAT)}"
            }
        } else {
            tv_my_vip.text = "开通VIP会员"
            tv_my_status.text = "解锁全部短剧"
            tv_my_btn.visibility = View.VISIBLE
            tv_my_btn.text = "立即开通"
        }
    }

    private fun initListener() {
        ll_my_login.setOnClickListener(this)
        tv_my_btn.setOnClickListener(this)
        tv_my_account.setOnClickListener(this)
        ll_my_record.setOnClickListener(this)
        ll_my_feed.setOnClickListener(this)
        ll_my_payrecord.setOnClickListener(this)
        ll_my_setting.setOnClickListener(this)
        ll_contactus.setOnClickListener(this)
    }

    private fun showAd() {
        MyExpressManager.instance.getAdView(object : ExpressAdCallback {
            override fun loadSuccess(adView: FrameLayout?) {
                adView?.let {
                    UIUtils.removeFromParent(it)
                    fl_ad.removeAllViews()
                    fl_ad.addView(it)
                }

                MyExpressManager.instance.preLoadAd()
            }

            override fun renderSuccess(adView: FrameLayout?): Boolean {
                adView?.let {
                    UIUtils.removeFromParent(it)
                    fl_ad.removeAllViews()
                    fl_ad.addView(it)
                }

                MyExpressManager.instance.preLoadAd()
                return false
            }

            override fun loadFail() {
            }
        })
    }

}