package com.duben.roseplaylet.ui.fragment

import android.Manifest
import android.os.Build
import android.os.Handler
import android.os.Looper
import android.view.LayoutInflater
import android.view.View
import android.widget.TextView
import androidx.core.content.ContextCompat
import androidx.recyclerview.widget.RecyclerView
import androidx.viewpager2.widget.ViewPager2
import com.amap.api.location.AMapLocationClient
import com.amap.api.location.AMapLocationClientOption
import com.amap.api.location.AMapLocationListener
import com.amap.api.maps.MapsInitializer
import com.duben.library.utils.nodoubleclick.AntiShake
import com.duben.roseplaylet.MintsApplication
import com.duben.roseplaylet.R
import com.duben.roseplaylet.common.AppConfig
import com.duben.roseplaylet.common.Constant
import com.duben.roseplaylet.mvp.model.*
import com.duben.roseplaylet.mvp.presenters.SquarePresenter
import com.duben.roseplaylet.mvp.views.SquareView
import com.duben.roseplaylet.ui.adapter.*
import com.duben.roseplaylet.ui.fragment.base.LazyLoadBaseFragment
import com.duben.roseplaylet.utils.AppPreferencesManager
import com.duben.roseplaylet.utils.LogUtil
import com.duben.roseplaylet.utils.PermissionXUtils
import com.duben.roseplaylet.utils.SpanUtils
import com.google.android.material.tabs.TabLayout
import com.google.android.material.tabs.TabLayoutMediator
import kotlinx.android.synthetic.main.fragment_square.*
import java.lang.reflect.Field

class SquareFragment : LazyLoadBaseFragment(), SquareView, View.OnClickListener {

    //声明AMapLocationClient类对象
    private var mLocationClient: AMapLocationClient? = null
    private var mLocationOption: AMapLocationClientOption? = null
    private var isFirstLocation = false
    private var haveLocationPermission = false

    private val squarePresenter by lazy { SquarePresenter() }

    private val bannerList = mutableListOf<BannerBean>()
    private val tabsData = mutableListOf<String>()
    private var vpAdapter: SquarePageAdapter? = null

    override fun getContentViewLayoutID() = R.layout.fragment_square

    override fun initViewsAndEvents() {
        initVp()

        initLocation()
        tv_location.setOnClickListener(this)
    }

    override fun onFragmentFirstVisible() {
        super.onFragmentFirstVisible()
        squarePresenter.attachView(this)
    }

    override fun onFragmentResume() {
        super.onFragmentResume()
        if (AppConfig.fragmentClickFlag == Constant.FRAGMENT_CLICK_TWO) {
            banner?.start()
            squarePresenter.topTabs()
        }
    }

    override fun onDestroy() {
        super.onDestroy()
        mLocationClient?.stopLocation()
        mLocationClient?.onDestroy()
        mLocationClient = null

        squarePresenter.detachView()
    }

    private fun initVp() {
        tabsData.addAll(arrayOf("附近", "推荐", "新人"))

        vpAdapter = SquarePageAdapter(tabsData, this)
        vp2.adapter = vpAdapter
        vp2.desensitization()
        TabLayoutMediator(tablayout, vp2) { tab, position ->
            // 初始化Tab
            tab.id = position
            if (tabsData.size > 0) {
                if (position == 0) {
                    tab.customView = getTabView(tabsData[position], true)
                } else {
                    tab.customView = getTabView(tabsData[position], false)
                }
            }
        }.attach()
        tablayout.addOnTabSelectedListener(object : TabLayout.OnTabSelectedListener {
            override fun onTabSelected(tab: TabLayout.Tab?) {
                tab?.let {
                    updateTab(it.customView, true)

                    showLocationTx()
                }
            }

            override fun onTabUnselected(tab: TabLayout.Tab?) {
                tab?.let {
                    updateTab(it.customView, false)
                }
            }

            override fun onTabReselected(tab: TabLayout.Tab?) {}
        })
        vp2.offscreenPageLimit = 100
    }

    private fun updateTab(view: View?, isSelected: Boolean) {
        view?.let {
            val text = it.findViewById<TextView>(R.id.item_tv)
            val bar = it.findViewById<View>(R.id.view_bar)
            if (isSelected) {
                text.setTextColor(ContextCompat.getColor(requireContext(), R.color.color_E4C46C))
                bar.visibility = View.VISIBLE
            } else {
                text.setTextColor(ContextCompat.getColor(requireContext(), R.color.color_959595))
                bar.visibility = View.INVISIBLE
            }
        }
    }

    private fun getTabView(text: String, isSelected: Boolean): View {
        val view = LayoutInflater.from(requireContext()).inflate(R.layout.item_square_tab, null)
        view.findViewById<TextView>(R.id.item_tv).text = text
        updateTab(view, isSelected)
        return view
    }

    override fun topTabsSuc(data: BannerList?) {
        if (data != null && data.list != null && data.list.size > 0) {
            bannerList.clear()
            for (i in 0 until data.list.size) {
                val bean = BannerBean()
                bean.title = data.list[i].topTabsReamrk
                bean.imageRes = data.list[i].typeImg
                bannerList.add(bean)
            }

            banner.addBannerLifecycleObserver(this)
                .setAdapter(ImageTitleAdapter(bannerList))
                .setOnBannerListener { data2, position ->
                    val banner = data.list[position]
                    if (banner.createType == 4) {
                    } else {
                        // TODO
                    }
                }
        }
    }

    private fun initLocation() {
        val permissions: List<String> = if (Build.VERSION.SDK_INT >= 29) {
            listOf(
                Manifest.permission.ACCESS_FINE_LOCATION,
                Manifest.permission.ACCESS_COARSE_LOCATION,
                Manifest.permission.ACCESS_BACKGROUND_LOCATION
            )
        } else {
            listOf(
                Manifest.permission.ACCESS_FINE_LOCATION,
                Manifest.permission.ACCESS_COARSE_LOCATION
            )
        }
        if (PermissionXUtils.checkLocationPermission(requireContext(), permissions)) {
            haveLocationPermission = true

            startLocation()
        } else {
            haveLocationPermission = false

            requestPermissions()

            tv_location.text = SpanUtils()
                .append("未开启")
                .setForegroundColor(ContextCompat.getColor(requireContext(), R.color.color_959595))
                .append(" 定位服务 ")
                .setForegroundColor(ContextCompat.getColor(requireContext(), R.color.color_E5C565))
                .append("无法计算你与其他用户的距离")
                .setForegroundColor(ContextCompat.getColor(requireContext(), R.color.color_959595))
                .create()
            showLocationTx()
        }
    }

    private fun showLocationTx() {
        if (haveLocationPermission) {
            tv_location.visibility = View.GONE
        } else {
            tv_location.visibility = View.VISIBLE
        }
    }

    private fun requestPermissions() {
        PermissionXUtils.requestLocationPermission(
            this, object : PermissionXUtils.Callback {
                override fun success() {
                    haveLocationPermission = true
                    showLocationTx()

                    AppPreferencesManager.get().put(Constant.FIRST_LOCATION, false)
                    AppPreferencesManager.get().put(Constant.NEED_REQUEST_LOCATION, true)
                    // 首次定位提示
                    if (!isFirstLocation) {
                        isFirstLocation = true
                        Handler(Looper.getMainLooper()).postDelayed({
                            showToast("正在开启定位...")
                        }, 300)
                    }
                    startLocation()
                }

                override fun fail() {
                    haveLocationPermission = false
                    // showMissingPermissionDialog("位置")
                    AppPreferencesManager.get().put(Constant.NEED_REQUEST_LOCATION, false)
                }
            }
        )
    }

    private fun startLocation() {
        // 高德地图合规设定
        MapsInitializer.updatePrivacyShow(context, true, true)
        MapsInitializer.updatePrivacyAgree(context, true)

        mLocationClient = AMapLocationClient(MintsApplication.getContext())
        mLocationOption = AMapLocationClientOption()

        mLocationOption?.let {
            // 设置定位模式为高精度模式
            it.locationMode = AMapLocationClientOption.AMapLocationMode.Hight_Accuracy
            //只定位一次
            it.isOnceLocation = false
            // 设置是否需要显示地址信息
            it.isNeedAddress = true
            /**
             * 设置是否优先返回GPS定位结果，如果30秒内GPS没有返回定位结果则进行网络定位
             * 注意：只有在高精度模式下的单次定位有效，其他方式无效
             */
            it.isGpsFirst = true
            // 设置是否开启缓存
            it.isLocationCacheEnable = true
            // 设置发送定位请求的时间间隔,最小值为1000，如果小于1000，按照1000算
            it.interval = 1000
        }

        mLocationClient?.let {
            // 设置定位监听
            it.setLocationListener(mapLocationListener)
            it.setLocationOption(mLocationOption)
            it.startLocation()
        }
    }

    private val mapLocationListener = AMapLocationListener { loc ->
        if (loc != null && loc.errorCode == 0) {
            // app全局使用
            val curLongitude = loc.longitude
            val curLatitude = loc.latitude
            val poiName = loc.poiName

            LogUtil.d("AAAAAA ------> curLongitude${curLongitude} curLatitude${curLatitude} poiName${poiName}")

            mLocationClient?.stopLocation()
        }
    }

    override fun getHallListSuc(list: BannerList) {

    }

    override fun getHallListFail() {
    }

    override fun autoListSuc(data: BannerList) {}

    override fun autoListFail() {}

    fun ViewPager2.desensitization() {
        //动态设置ViewPager2 灵敏度
        try {
            val recyclerViewField: Field = ViewPager2::class.java.getDeclaredField("mRecyclerView")
            recyclerViewField.isAccessible = true
            val recyclerView = recyclerViewField.get(this) as RecyclerView
            val touchSlopField: Field = RecyclerView::class.java.getDeclaredField("mTouchSlop")
            touchSlopField.isAccessible = true
            val touchSlop = touchSlopField.get(recyclerView) as Int
            touchSlopField.set(recyclerView, touchSlop * 3) //6 is empirical value
        } catch (ignore: java.lang.Exception) {
        }
    }

    override fun onClick(v: View?) {
        if (AntiShake.check(v?.id)) return

        when (v?.id) {
            R.id.tv_location -> {
                requestPermissions()
            }
        }
    }
}