package com.duben.roseplaylet.ui.fragment

import android.os.Bundle
import android.view.LayoutInflater
import androidx.fragment.app.Fragment
import com.duben.roseplaylet.R
import com.duben.roseplaylet.ad.express.MainExpressManager
import com.duben.roseplaylet.common.Constant
import com.duben.roseplaylet.mvp.model.*
import com.duben.roseplaylet.ui.fragment.base.LazyLoadBaseFragment
import com.scwang.smartrefresh.layout.api.RefreshLayout
import com.scwang.smartrefresh.layout.listener.OnLoadMoreListener
import com.duben.roseplaylet.common.AppConfig
import com.duben.roseplaylet.mvp.presenters.SquarePresenter
import com.duben.roseplaylet.mvp.views.SquareView
import com.duben.roseplaylet.ui.adapter.SquareAdapter
import kotlinx.android.synthetic.main.fragment_square_list.*

class SquareListFragment : LazyLoadBaseFragment(), SquareView, OnLoadMoreListener {
    companion object {
        private const val SQUARE_TYPE = "SQUARE_TYPE"

        fun newInstance(): Fragment {
            val args = Bundle()
            val fragment = SquareListFragment()
            fragment.arguments = args
            return fragment
        }
    }

    private val squarePresenter by lazy { SquarePresenter() }

    private var hotPage = 1 // 分页
    private var hotPageSize = Constant.PAGE_SIZE // 分页

    private lateinit var squareAdapter: SquareAdapter

    override fun getContentViewLayoutID() = R.layout.fragment_square_list

    override fun initViewsAndEvents() {
        squarePresenter.attachView(this)

        initRecy()
        initListener()

        arguments?.let {}
    }

    override fun onResume() {
        super.onResume()
        hotPageSize = 15
        onRefresh()
    }

    override fun onDestroy() {
        super.onDestroy()
        squarePresenter.detachView()
    }

    fun onRefresh() {
        if (AppConfig.fragmentClickFlag == Constant.FRAGMENT_CLICK_TWO) {
            hotPage = 1
            srl_square.resetNoMoreData()
            squarePresenter.autoList()
        }
    }

    override fun onLoadMore(refreshLayout: RefreshLayout) {
        hotPage = ++hotPage
        squarePresenter.autoList()
    }

    private fun initRecy() {
        squareAdapter = SquareAdapter()
        squareAdapter.animationEnable = false
        rcy_square_list.adapter = squareAdapter
        val emptyView =
            LayoutInflater.from(requireContext()).inflate(R.layout.item_empty_data, null)
        squareAdapter.setEmptyView(emptyView)
    }

    private fun initListener() {
        srl_square.setEnableRefresh(false)
        srl_square.setOnLoadMoreListener(this)
    }

    override fun onDestroyView() {
        super.onDestroyView()
        MainExpressManager.instance.destroy()
    }

    override fun autoListSuc(data: BannerList) {
        if (activity == null || requireActivity().isFinishing) {
            srl_square?.finishRefresh(true)
            return
        }

        if (::squareAdapter.isInitialized) {
            if (hotPage == 1) {
                srl_square.finishRefresh(true)
                squareAdapter.setNewInstance(data.list)
            } else {
                if (data.list.size < hotPageSize) {
                    srl_square.finishLoadMoreWithNoMoreData()
                } else {
                    srl_square.finishLoadMore()
                }
                squareAdapter.addData(data.list)
            }
        }
    }

    override fun autoListFail() {}

    override fun topTabsSuc(data: BannerList?) {}
    override fun getHallListSuc(list: BannerList) {}
    override fun getHallListFail() {}
}