package com.duben.roseplaylet.ui.preview;

import android.annotation.SuppressLint;
import android.os.Bundle;
import android.view.View;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.fragment.app.Fragment;
import androidx.fragment.app.FragmentManager;
import androidx.lifecycle.Lifecycle;
import androidx.viewpager2.adapter.FragmentStateAdapter;
import androidx.viewpager2.widget.ViewPager2;

import com.duben.roseplaylet.R;
import com.duben.roseplaylet.ui.activitys.base.BaseActivity;

import java.util.ArrayList;
import java.util.List;

/**
 * 图片查看与保存Activity
 * <p>
 * 需传入string类型的集合
 *
 * @author zhangqin
 * @date 2017/8/7
 */
public class ImagePreviewActivity extends BaseActivity {

    public static final String IMAGE_PREVIEW_DATA = "data";
    public static final String IMAGE_PREVIEW_POSITION = "position";

    private ViewPager2 viewPager;
    private TextView mPosition;

    private List<String> mImageList = new ArrayList<>();
    private int position;

    private ShowImagePagerAdapter mShowImagePagerAdapter;

    /**
     * 指示器
     *
     * @param position
     * @param size
     */
    @SuppressLint("SetTextI18n")
    public void setPosText(int position, int size) {
        mPosition.setText(position + "/" + size);
    }

    @Override
    protected int getContentViewLayoutID() {
        return R.layout.quick_activity_image_preview;
    }

    @Override
    protected void getBundleExtras(Bundle extras) {
        super.getBundleExtras(extras);
        if (extras != null) {
            mImageList = this.getIntent().getStringArrayListExtra(IMAGE_PREVIEW_DATA);
            position = this.getIntent().getIntExtra(IMAGE_PREVIEW_POSITION, 0);
        } else {
            showToast("参数获取失败");
            finish();
        }
    }

    @Override
    protected void initViewsAndEvents() {
        viewPager = findViewById(R.id.viewpager);
        mPosition = findViewById(R.id.position);

        mShowImagePagerAdapter = new ShowImagePagerAdapter(getSupportFragmentManager(), getLifecycle());
        viewPager.setAdapter(mShowImagePagerAdapter);
        viewPager.setCurrentItem(position);

        if (mImageList.size() == 1) {
            mPosition.setVisibility(View.GONE);
        } else {
            mPosition.setVisibility(View.VISIBLE);
        }

        setPosText(position + 1, mImageList.size());

        viewPager.registerOnPageChangeCallback(new ViewPager2.OnPageChangeCallback() {
            @Override
            public void onPageSelected(int position) {
                setPosText(position + 1, mImageList.size());
            }
        });
    }

    @Override
    protected boolean isApplyKitKatTranslucency() {
        return false;
    }

    private class ShowImagePagerAdapter extends FragmentStateAdapter {

        public ShowImagePagerAdapter(@NonNull FragmentManager fragmentManager, @NonNull Lifecycle lifecycle) {
            super(fragmentManager, lifecycle);
        }

        @NonNull
        @Override
        public Fragment createFragment(int position) {
            return ImagePreviewFragment.newInstance(mImageList.get(position));
        }

        @Override
        public int getItemCount() {
            return mImageList.size();
        }
    }
}
