package com.duben.roseplaylet.ui.preview;

import android.os.Bundle;
import android.view.View;
import android.widget.ImageView;
import android.widget.ProgressBar;

import androidx.fragment.app.Fragment;

import com.duben.roseplaylet.R;
import com.duben.roseplaylet.ui.fragment.base.LazyLoadBaseFragment;
import com.duben.roseplaylet.ui.preview.imageloader.ImageLoader;
import com.duben.roseplaylet.ui.preview.imageloader.progress.OnProgressListener;
import com.google.android.material.bottomsheet.BottomSheetDialogFragment;

/**
 * 描述：显示图片
 *
 * @author zhangqin
 * @date 2017/8/7
 */
public class ImagePreviewFragment extends LazyLoadBaseFragment {

    private ImageView image;
    private ProgressBar progressBar;

    private String url;

    public static Fragment newInstance(String s) {
        Bundle bundle = new Bundle();
        bundle.putString("url", s);
        ImagePreviewFragment fragment = new ImagePreviewFragment();
        fragment.setArguments(bundle);
        return fragment;
    }

    private void initSaveImageDialog() {
        BottomSheetDialogFragment bottomSheetDialogFragment = new BottomDialogImagePreviewFragment
                .Builder()
                .setOnClickListener(new BottomDialogImagePreviewFragment.OnDialogClickListener() {
                    @Override
                    public void save() {
                        saveImage();
                    }

                    @Override
                    public void exit() {

                    }
                })
                .builder();
        bottomSheetDialogFragment.show(getChildFragmentManager(), "preview_image");
    }

    /**
     * 保存图片
     */
    public void saveImage() {
//        if (FileUtils.getRootDirectory() != null) {
//            try {
//                String filePath = QuickConstants.SAVE_REAL_PATH + url.substring(url.lastIndexOf("/"));
//                FileUtils.createAttachFile(filePath, url.substring(url.lastIndexOf("/")));
//                BitmapDrawable bitmapDrawable = (BitmapDrawable) image.getDrawable();
//                byte[] bitmap2Bytes = ImageUtils.bitmap2Bytes(bitmapDrawable.getBitmap(), Bitmap.CompressFormat.JPEG, 100);
//                if (FileIOUtils.writeFileFromBytesByStream(filePath, bitmap2Bytes)) {
//                    showToast("保存成功");
//                    Uri contentUri = Uri.fromFile(new File(filePath));
//                    Intent mediaScanIntent = new Intent(Intent.ACTION_MEDIA_SCANNER_SCAN_FILE, contentUri);
//                    mContext.sendBroadcast(mediaScanIntent);
//                } else {
//                    showToast("保存失败");
//                }
//            } catch (Exception e) {
//                showToast(e.getMessage());
//            }
//        } else {
//            showToast("内存卡不可用");
//        }
    }

    @Override
    public void onFragmentFirstVisible() {
        super.onFragmentFirstVisible();

        image = getView().findViewById(R.id.image);
        progressBar = getView().findViewById(R.id.progressBar);

        image.setOnLongClickListener(view -> {
//            initSaveImageDialog();
            return false;
        });
        image.setOnClickListener(v -> {
            requireActivity().finish();
        });

        loadImage();
    }

    private void loadImage() {
        new ImageLoader.Builder()
                .setImagePath(url)
                .build(image)
                .loadImage()
                .setOnProgressListener(new OnProgressListener() {
                    @Override
                    public void onLoading(int progress) {
                        progressBar.setVisibility(View.VISIBLE);
                        progressBar.setProgress(progress);
                    }

                    @Override
                    public void onLoadSuccess() {
                        progressBar.setVisibility(View.GONE);
                    }
                });

    }

    @Override
    protected void initViewsAndEvents() {
        if (getArguments() != null) {
            url = getArguments().getString("url");
        }
    }

    @Override
    protected int getContentViewLayoutID() {
        return R.layout.quick_fragment_image_preview;
    }
}
