package com.duben.roseplaylet.ui.widgets.nineImage

import android.content.Context
import android.util.DisplayMetrics
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.view.WindowManager
import android.widget.ImageView
import android.widget.RelativeLayout
import androidx.recyclerview.widget.RecyclerView
import com.bumptech.glide.Glide
import com.bumptech.glide.request.RequestOptions
import com.duben.roseplaylet.R

class AddImagesAdapter(
    mContext: Context,
    limtNum: Int,
    addImgs: Int,
    addCloseImgs: Int,
    addErrImgs: Int,
    addLimitNums: Int
) :
    RecyclerView.Adapter<RecyclerView.ViewHolder>() {
    var mContext: Context
    var limtNum = 5
    var addImgs = -1
    var addCloseImgs = -1
    var addErrImgs = -1
    var addLimitNums = 9
    var addImages = ArrayList<AddImagesInfo>()

    init {
        this.mContext = mContext
        this.limtNum = limtNum
        this.addCloseImgs = addCloseImgs
        this.addImgs = addImgs
        this.addErrImgs = addErrImgs
        this.addLimitNums = addLimitNums
    }

    var listener: OnAddClickListener? = null

    fun setOnAddListener(listener: OnAddClickListener) {
        this.listener = listener
    }

    fun getHeight(): Int {
        return (getScreenWidth(mContext)) / 5
    }

    private fun dip2px(context: Context, dp: Int): Int {
        val density = context.resources.displayMetrics.density
        return (dp * density + 0.5).toInt()
    }

    private fun getScreenWidth(context: Context): Int {
        val wm = context
            .getSystemService(Context.WINDOW_SERVICE) as WindowManager
        val outMetrics = DisplayMetrics()
        wm.defaultDisplay.getMetrics(outMetrics)
        return outMetrics.widthPixels
    }


    override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): RecyclerView.ViewHolder {
        val view = LayoutInflater.from(parent.context)
            .inflate(R.layout.item_add_images, parent, false)
        return AddImagesViewHolder(view)
    }

    override fun onBindViewHolder(holder: RecyclerView.ViewHolder, position: Int) {
        val ktHolder = holder as AddImagesViewHolder
        if (addImgs != -1) {
            ktHolder.ivAdd.setBackgroundResource(addImgs)
        } else {
            ktHolder.ivAdd.setBackgroundResource(R.drawable.iv_add_images_add)
        }
        if (addCloseImgs != -1) {
            ktHolder.ivClose.setBackgroundResource(addCloseImgs)
        } else {
            ktHolder.ivClose.setBackgroundResource(R.drawable.iv_add_img_close)
        }

        ktHolder.rlClose.setOnClickListener {
            processDelete(getItem(position))
            if (listener != null) listener!!.onDeleteClick(position)
        }

        ktHolder.rlAdd.setOnClickListener {
            if (listener != null) listener!!.onAddClick(ktHolder.rlAdd, getItem(position))
        }

        ktHolder.ivContent.setOnClickListener {
            if (listener != null) listener!!.onAddClick(ktHolder.ivContent, getItem(position))
        }

        if (getItem(position)!!.type == "2") {
            ktHolder.ivContent.visibility = View.VISIBLE
            ktHolder.rlClose.visibility = View.VISIBLE
            ktHolder.rlAdd.visibility = View.GONE
            if (addErrImgs != -1) {
                val requestOptions = RequestOptions().error(addErrImgs)
                Glide.with(mContext).load(getItem(position)!!.imgUrl).apply(requestOptions)
                    .into(ktHolder.ivContent)
            } else {
                val requestOptions = RequestOptions().error(R.drawable.iv_add_err)
                Glide.with(mContext).load(getItem(position)!!.imgUrl).apply(requestOptions)
                    .into(ktHolder.ivContent)
            }

        } else {
            ktHolder.ivContent.visibility = View.GONE
            ktHolder.rlAdd.visibility = View.VISIBLE
            ktHolder.rlClose.visibility = View.GONE
        }

    }

    private fun processDelete(item: AddImagesInfo?) {
        addImages.remove(item)
        if (!hasAdd()) {
            addItem(AddImagesInfo(R.drawable.iv_add_images_add, "1"))
        } else {
            notifyDataSetChanged()
        }

    }

    fun addItems(lists: List<AddImagesInfo>) {
        processData(lists)
    }

    fun addItem(info: AddImagesInfo) {
        addImages.add(info)
        notifyDataSetChanged()
    }

    fun getItems(): ArrayList<AddImagesInfo> {
        var hasAdd = false
        for (addImage in this.addImages) {
            if (addImage.type != "2") {
                hasAdd = true
            }
        }
        if (hasAdd) {
            val items = this.addImages
            items.removeLast()
            return items
        }
        return this.addImages
    }

    private fun processData(addDatas: List<AddImagesInfo>) {
        val newList = processEnd(getItems())
//        if (newList.size >= limtNum) return
        if (newList.size + addDatas.size < addLimitNums) {
            newList.addAll(addDatas)
            newList.add(AddImagesInfo("", "1"))
        } else {
            newList.addAll(addDatas.subList(0, addLimitNums - newList.size))
        }
        notifyDataSetChanged()

    }

    private fun processEnd(addDatas: ArrayList<AddImagesInfo>): ArrayList<AddImagesInfo> {
        val iterator = addDatas.iterator()
        while (iterator.hasNext()) {
            val info = iterator.next()
            if (info.type == "1") iterator.remove()
        }
        return addDatas
    }

    private fun hasAdd(): Boolean {
        getItems()!!.forEach {
            if (it.type == "1") {
                return true
            }
        }
        return false
    }


    override fun getItemCount(): Int {
        return addImages.size
    }

    fun getItem(position: Int): AddImagesInfo? {
        if (position < addImages.size) {
            return this.addImages[position]
        }
        return null
    }

    interface OnAddClickListener {
        fun onAddClick(view: View?, item: AddImagesInfo?)
        fun onDeleteClick(position: Int)
    }

    class AddImagesViewHolder(view: View) : RecyclerView.ViewHolder(view) {
        val ivAdd: ImageView = view.findViewById(R.id.iv_add)
        val ivClose: ImageView = view.findViewById(R.id.iv_close)
        val rlClose: RelativeLayout = view.findViewById(R.id.rl_close)
        val ivContent: ImageView = view.findViewById(R.id.iv_content)
        val rlAdd: RelativeLayout = view.findViewById(R.id.rl_add)
    }
}