package com.duben.roseplaylet.ui.widgets.pick

import android.app.Activity
import android.app.Dialog
import android.text.TextUtils
import android.view.Gravity
import android.view.View
import android.view.WindowManager
import android.widget.CheckBox
import android.widget.TextView
import androidx.recyclerview.widget.LinearLayoutManager
import androidx.recyclerview.widget.RecyclerView
import com.duben.roseplaylet.R
import com.duben.roseplaylet.mvp.model.PickBean

/**
 *  @author Assen
 *  @date 2023/12/27
 *  @desc
 */
class MultiPickDialog(
    val activity: Activity,
    title: String,
    var data: MutableList<PickBean>,
    defaultValue: String = ""
) :
    Dialog(activity, R.style.dialog) {

    private var mOnPickResultListener: OnPickResultListener? = null

    init {
        setContentView(R.layout.dialog_pick)
        // 设置window属性
        val lp: WindowManager.LayoutParams = window!!.attributes
        lp.gravity = Gravity.CENTER
        lp.width = WindowManager.LayoutParams.MATCH_PARENT
        lp.windowAnimations = R.style.DialogAnimFade
        window!!.attributes = lp

        // 设置外部不可关闭
//        setCancelable(false)
//        setCanceledOnTouchOutside(false)
//        setOnKeyListener { _, i, _ -> i == KeyEvent.KEYCODE_BACK }
        findViewById<View>(R.id.ll_btn).visibility = View.VISIBLE

        findViewById<TextView>(R.id.tv_title).text = title
        findViewById<View>(R.id.tv_cancel).setOnClickListener {
            dismiss()
        }
        findViewById<View>(R.id.tv_confirm).setOnClickListener {
            dismiss()
            val sb = java.lang.StringBuilder()
            for (datum in data) {
                if (datum.isChecked) {
                    sb.append("$datum,")
                }
            }
            mOnPickResultListener?.onPickResult(sb.toString())
        }
        val rv = findViewById<RecyclerView>(R.id.rv_pick)

        rv.layoutManager = LinearLayoutManager(activity)
        val multiPickAdapter = MultiPickAdapter()
        multiPickAdapter.setOnItemClickListener { adapter, view, position ->
            val childViewHolder = rv.getChildViewHolder(view)
            data[position].isChecked = !data[position].isChecked
            childViewHolder.itemView.findViewById<CheckBox>(R.id.cb).isChecked =
                data[position].isChecked
        }
        multiPickAdapter.setNewInstance(data)
        rv.adapter = multiPickAdapter

        if (!TextUtils.isEmpty(defaultValue)) {
            for (i in 0 until data.size) {
                if (TextUtils.equals(defaultValue, data[i].title)) {
                    data[i].isChecked = true
                    rv.scrollToPosition(i)
                    multiPickAdapter.notifyItemChanged(i)
                    break
                }
            }
        }
    }

    fun setPickResultListener(onPickResultListener: OnPickResultListener) {
        mOnPickResultListener = onPickResultListener
    }

    interface OnPickResultListener {
        fun onPickResult(result: String)
    }
}